@@ -1,6 +1,6 @@
 /* Everything about catch/throw catchpoints, for GDB.
 
-   Copyright (C) 1986-2022 Free Software Foundation, Inc.
+   Copyright (C) 1986-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,7 +17,6 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 #include "arch-utils.h"
 #include <ctype.h>
 #include "breakpoint.h"
@@ -87,20 +86,13 @@ struct exception_catchpoint : public code_breakpoint
 
   void re_set () override;
   enum print_stop_action print_it (const bpstat *bs) const override;
-  bool print_one (bp_location **) const override;
+  bool print_one (const bp_location **) const override;
   void print_mention () const override;
   void print_recreate (struct ui_file *fp) const override;
   void print_one_detail (struct ui_out *) const override;
   void check_status (struct bpstat *bs) override;
   struct bp_location *allocate_location () override;
 
-  /* FIXME this is temporary - until ordinary breakpoints have been
-     converted.  */
-  int resources_needed (const struct bp_location *) override
-  {
-    return 1;
-  }
-
   /* The kind of exception catchpoint.  */
 
   enum exception_event_kind kind;
@@ -167,15 +159,13 @@ fetch_probe_arguments (struct value **arg0, struct value **arg1)
 void
 exception_catchpoint::check_status (struct bpstat *bs)
 {
-  struct exception_catchpoint *self
-    = (struct exception_catchpoint *) bs->breakpoint_at;
   std::string type_name;
 
   this->breakpoint::check_status (bs);
-  if (bs->stop == 0)
+  if (!bs->stop)
     return;
 
-  if (self->pattern == NULL)
+  if (this->pattern == NULL)
     return;
 
   const char *name = nullptr;
@@ -199,8 +189,8 @@ exception_catchpoint::check_status (struct bpstat *bs)
 
   if (name != nullptr)
     {
-      if (self->pattern->exec (name, 0, NULL, 0) != 0)
-	bs->stop = 0;
+      if (this->pattern->exec (name, 0, NULL, 0) != 0)
+	bs->stop = false;
     }
 }
 
@@ -268,7 +258,7 @@ exception_catchpoint::print_it (const bpstat *bs) const
 }
 
 bool
-exception_catchpoint::print_one (bp_location **last_loc) const
+exception_catchpoint::print_one (const bp_location **last_loc) const
 {
   struct value_print_options opts;
   struct ui_out *uiout = current_uiout;
