@@ -1,6 +1,6 @@
 /* Support for printing C++ values for GDB, the GNU debugger.
 
-   Copyright (C) 1986-2023 Free Software Foundation, Inc.
+   Copyright (C) 1986-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,7 +17,6 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 #include "gdbsupport/gdb_obstack.h"
 #include "symtab.h"
 #include "gdbtypes.h"
@@ -129,7 +128,7 @@ cp_print_value_fields (struct value *val, struct ui_file *stream,
   int fields_seen = 0;
   static int last_set_recurse = -1;
 
-  struct type *type = check_typedef (value_type (val));
+  struct type *type = check_typedef (val->type ());
 
   if (recurse == 0)
     {
@@ -189,11 +188,11 @@ cp_print_value_fields (struct value *val, struct ui_file *stream,
       vptr_fieldno = get_vptr_fieldno (type, &vptr_basetype);
       for (i = n_baseclasses; i < len; i++)
 	{
-	  const gdb_byte *valaddr = value_contents_for_printing (val).data ();
+	  const gdb_byte *valaddr = val->contents_for_printing ().data ();
 
 	  /* If requested, skip printing of static fields.  */
 	  if (!options->static_field_print
-	      && field_is_static (&type->field (i)))
+	      && type->field (i).is_static ())
 	    continue;
 
 	  if (fields_seen)
@@ -227,7 +226,7 @@ cp_print_value_fields (struct value *val, struct ui_file *stream,
 
 	  annotate_field_begin (type->field (i).type ());
 
-	  if (field_is_static (&type->field (i)))
+	  if (type->field (i).is_static ())
 	    {
 	      gdb_puts ("static ", stream);
 	      fprintf_symbol (stream,
@@ -258,31 +257,31 @@ cp_print_value_fields (struct value *val, struct ui_file *stream,
 	    }
 	  annotate_field_value ();
 
-	  if (!field_is_static (&type->field (i))
-	      && TYPE_FIELD_PACKED (type, i))
+	  if (!type->field (i).is_static ()
+	      && type->field (i).is_packed ())
 	    {
 	      struct value *v;
 
 	      /* Bitfields require special handling, especially due to
 		 byte order problems.  */
-	      if (TYPE_FIELD_IGNORE (type, i))
+	      if (type->field (i).is_ignored ())
 		{
 		  fputs_styled ("<optimized out or zero length>",
 				metadata_style.style (), stream);
 		}
-	      else if (value_bits_synthetic_pointer
-			 (val, type->field (i).loc_bitpos (),
-			  TYPE_FIELD_BITSIZE (type, i)))
+	      else if (val->bits_synthetic_pointer
+		       (type->field (i).loc_bitpos (),
+			type->field (i).bitsize ()))
 		{
 		  fputs_styled (_("<synthetic pointer>"),
 				metadata_style.style (), stream);
 		}
 	      else
 		{
-		  opts->deref_ref = 0;
+		  opts->deref_ref = false;
 
 		  v = value_field_bitfield (type, i, valaddr,
-					    value_embedded_offset (val), val);
+					    val->embedded_offset (), val);
 
 		  common_val_print (v, stream, recurse + 1,
 				    opts, current_language);
@@ -290,12 +289,12 @@ cp_print_value_fields (struct value *val, struct ui_file *stream,
 	    }
 	  else
 	    {
-	      if (TYPE_FIELD_IGNORE (type, i))
+	      if (type->field (i).is_ignored ())
 		{
 		  fputs_styled ("<optimized out or zero length>",
 				metadata_style.style (), stream);
 		}
-	      else if (field_is_static (&type->field (i)))
+	      else if (type->field (i).is_static ())
 		{
 		  try
 		    {
@@ -321,7 +320,7 @@ cp_print_value_fields (struct value *val, struct ui_file *stream,
 		    {
 		      CORE_ADDR addr;
 
-		      i_offset += value_embedded_offset (val);
+		      i_offset += val->embedded_offset ();
 		      addr = extract_typed_address (valaddr + i_offset, i_type);
 		      print_function_pointer_address (opts,
 						      type->arch (),
@@ -330,8 +329,8 @@ cp_print_value_fields (struct value *val, struct ui_file *stream,
 		}
 	      else
 		{
-		  struct value *v = value_primitive_field (val, 0, i, type);
-		  opts->deref_ref = 0;
+		  struct value *v = val->primitive_field (0, i, type);
+		  opts->deref_ref = false;
 		  common_val_print (v, stream, recurse + 1, opts,
 				    current_language);
 		}
@@ -381,6 +380,32 @@ cp_print_value_fields (struct value *val, struct ui_file *stream,
   gdb_printf (stream, "}");
 }
 
+/* A wrapper for cp_print_value_fields that tries to apply a
+   pretty-printer first.  */
+
+static void
+cp_print_value_fields_pp (struct value *val,
+			  struct ui_file *stream,
+			  int recurse,
+			  const struct value_print_options *options,
+			  struct type **dont_print_vb,
+			  int dont_print_statmem)
+{
+  int result = 0;
+
+  /* Attempt to run an extension language pretty-printer if
+     possible.  */
+  if (!options->raw)
+    result
+      = apply_ext_lang_val_pretty_printer (val, stream,
+					   recurse, options,
+					   current_language);
+
+  if (!result)
+    cp_print_value_fields (val, stream, recurse, options, dont_print_vb,
+			   dont_print_statmem);
+}
+
 /* Special val_print routine to avoid printing multiple copies of
    virtual baseclasses.  */
 
@@ -389,13 +414,13 @@ cp_print_value (struct value *val, struct ui_file *stream,
 		int recurse, const struct value_print_options *options,
 		struct type **dont_print_vb)
 {
-  struct type *type = check_typedef (value_type (val));
-  CORE_ADDR address = value_address (val);
+  struct type *type = check_typedef (val->type ());
+  CORE_ADDR address = val->address ();
   struct type **last_dont_print
     = (struct type **) obstack_next_free (&dont_print_vb_obstack);
   struct obstack tmp_obstack = dont_print_vb_obstack;
   int i, n_baseclasses = TYPE_N_BASECLASSES (type);
-  const gdb_byte *valaddr = value_contents_for_printing (val).data ();
+  const gdb_byte *valaddr = val->contents_for_printing ().data ();
 
   if (dont_print_vb == 0)
     {
@@ -432,7 +457,7 @@ cp_print_value (struct value *val, struct ui_file *stream,
       try
 	{
 	  boffset = baseclass_offset (type, i, valaddr,
-				      value_embedded_offset (val),
+				      val->embedded_offset (),
 				      address, val);
 	}
       catch (const gdb_exception_error &ex)
@@ -461,7 +486,7 @@ cp_print_value (struct value *val, struct ui_file *stream,
 		  base_val = value_from_contents_and_address (baseclass,
 							      buf.data (),
 							      address + boffset);
-		  baseclass = value_type (base_val);
+		  baseclass = base_val->type ();
 		  boffset = 0;
 		}
 	      else
@@ -493,27 +518,16 @@ cp_print_value (struct value *val, struct ui_file *stream,
 	val_print_invalid_address (stream);
       else
 	{
-	  int result = 0;
-
 	  if (!val_print_check_max_depth (stream, recurse, options,
 					  current_language))
 	    {
-	      struct value *baseclass_val = value_primitive_field (val, 0,
-								   i, type);
+	      struct value *baseclass_val = val->primitive_field (0,
+								  i, type);
 
-	      /* Attempt to run an extension language pretty-printer on the
-		 baseclass if possible.  */
-	      if (!options->raw)
-		result
-		  = apply_ext_lang_val_pretty_printer (baseclass_val, stream,
-						       recurse, options,
-						       current_language);
-
-	      if (!result)
-		cp_print_value_fields (baseclass_val, stream, recurse, options,
-				       ((struct type **)
-					obstack_base (&dont_print_vb_obstack)),
-				       0);
+	      cp_print_value_fields_pp
+		(baseclass_val, stream, recurse, options,
+		 (struct type **) obstack_base (&dont_print_vb_obstack),
+		 0);
 	    }
 	}
       gdb_puts (", ", stream);
@@ -550,7 +564,7 @@ cp_print_static_field (struct type *type,
 {
   struct value_print_options opts;
 
-  if (value_entirely_optimized_out (val))
+  if (val->entirely_optimized_out ())
     {
       val_print_optimized_out (val, stream);
       return;
@@ -560,7 +574,7 @@ cp_print_static_field (struct type *type,
   if (real_type->code () == TYPE_CODE_STRUCT)
     {
       CORE_ADDR *first_dont_print;
-      CORE_ADDR addr = value_address (val);
+      CORE_ADDR addr = val->address ();
       int i;
 
       first_dont_print
@@ -581,7 +595,7 @@ cp_print_static_field (struct type *type,
 
       obstack_grow (&dont_print_statmem_obstack, (char *) &addr,
 		    sizeof (CORE_ADDR));
-      cp_print_value_fields (val, stream, recurse, options, NULL, 1);
+      cp_print_value_fields_pp (val, stream, recurse, options, nullptr, 1);
       return;
     }
 
@@ -613,7 +627,7 @@ cp_print_static_field (struct type *type,
     }
 
   opts = *options;
-  opts.deref_ref = 0;
+  opts.deref_ref = false;
   common_val_print (val, stream, recurse, &opts, current_language);
 }
 
@@ -637,7 +651,7 @@ cp_find_class_member (struct type **self_p, int *fieldno,
   for (i = TYPE_N_BASECLASSES (self); i < len; i++)
     {
       field &f = self->field (i);
-      if (field_is_static (&f))
+      if (f.is_static ())
 	continue;
       LONGEST bitpos = f.loc_bitpos ();
 
@@ -737,13 +751,13 @@ test_print_fields (gdbarch *arch)
     {
       f = append_composite_type_field_raw (the_struct, "A", bool_type);
       f->set_loc_bitpos (1);
-      FIELD_BITSIZE (*f) = 1;
+      f->set_bitsize (1);
       f = append_composite_type_field_raw (the_struct, "B", uint8_type);
       f->set_loc_bitpos (3);
-      FIELD_BITSIZE (*f) = 3;
+      f->set_bitsize (3);
       f = append_composite_type_field_raw (the_struct, "C", bool_type);
       f->set_loc_bitpos (7);
-      FIELD_BITSIZE (*f) = 1;
+      f->set_bitsize (1);
     }
   /* According to the logic commented in "make_gdb_type_struct ()" of
    * target-descriptions.c, bit positions are numbered differently for
@@ -752,18 +766,18 @@ test_print_fields (gdbarch *arch)
     {
       f = append_composite_type_field_raw (the_struct, "A", bool_type);
       f->set_loc_bitpos (30);
-      FIELD_BITSIZE (*f) = 1;
+      f->set_bitsize (1);
       f = append_composite_type_field_raw (the_struct, "B", uint8_type);
       f->set_loc_bitpos (26);
-      FIELD_BITSIZE (*f) = 3;
+      f->set_bitsize (3);
       f = append_composite_type_field_raw (the_struct, "C", bool_type);
       f->set_loc_bitpos (24);
-      FIELD_BITSIZE (*f) = 1;
+      f->set_bitsize (1);
     }
 
-  value *val = allocate_value (the_struct);
-  gdb_byte *contents = value_contents_writeable (val).data ();
-  store_unsigned_integer (contents, value_enclosing_type (val)->length (),
+  value *val = value::allocate (the_struct);
+  gdb_byte *contents = val->contents_writeable ().data ();
+  store_unsigned_integer (contents, val->enclosing_type ()->length (),
 			  gdbarch_byte_order (arch), 0xe9);
 
   string_file out;
