@@ -1,7 +1,7 @@
 /* Private implementation details of interface between gdb and its
    extension languages.
 
-   Copyright (C) 2014-2023 Free Software Foundation, Inc.
+   Copyright (C) 2014-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -119,6 +119,10 @@ struct extension_language_ops
      This method is required.  */
   int (*initialized) (const struct extension_language_defn *);
 
+  /* Called just before GDB exits.  This shuts down the extension
+     language.  This can be NULL.  */
+  void (*shutdown) (const struct extension_language_defn *);
+
   /* Process a sequence of commands embedded in GDB's own scripting language.
      E.g.,
      python
@@ -137,7 +141,7 @@ struct extension_language_ops
 			       struct ext_lang_type_printers *);
 
   /* Try to pretty-print TYPE.  If successful the pretty-printed type is
-     stored in *PRETTIED_TYPE, and the caller must free it.
+     stored in *PRETTIED_TYPE.
      Returns EXT_LANG_RC_OK upon success, EXT_LANG_RC_NOP if the type
      is not recognized, and EXT_LANG_RC_ERROR if an error was encountered.
      This function has a bit of a funny name, since it actually applies
@@ -146,7 +150,8 @@ struct extension_language_ops
   enum ext_lang_rc (*apply_type_printers)
     (const struct extension_language_defn *,
      const struct ext_lang_type_printers *,
-     struct type *, char **prettied_type);
+     struct type *,
+     gdb::unique_xmalloc_ptr<char> *prettied_type);
 
   /* Called after a type has been printed to give the type pretty-printer
      mechanism an opportunity to clean up.  */
@@ -179,7 +184,7 @@ struct extension_language_ops
      or SCR_BT_COMPLETED on success.  */
   enum ext_lang_bt_status (*apply_frame_filter)
     (const struct extension_language_defn *,
-     frame_info_ptr frame, frame_filter_flags flags,
+     const frame_info_ptr &frame, frame_filter_flags flags,
      enum ext_lang_frame_args args_type,
      struct ui_out *out, int frame_low, int frame_high);
 
@@ -255,13 +260,13 @@ struct extension_language_ops
      CONTENTS is the contents of the file.  This should either return
      colorized (using ANSI terminal escapes) version of the contents,
      or an empty option.  */
-  gdb::optional<std::string> (*colorize) (const std::string &name,
+  std::optional<std::string> (*colorize) (const std::string &name,
 					  const std::string &contents);
 
   /* Colorize a single line of disassembler output, CONTENT.  This should
      either return colorized (using ANSI terminal escapes) version of the
      contents, or an empty optional.  */
-  gdb::optional<std::string> (*colorize_disasm) (const std::string &content,
+  std::optional<std::string> (*colorize_disasm) (const std::string &content,
 						 gdbarch *gdbarch);
 
   /* Print a single instruction from ADDRESS in architecture GDBARCH.  INFO
@@ -275,9 +280,16 @@ struct extension_language_ops
      If no instruction can be disassembled then return an empty value and
      other extension languages will get a chance to perform the
      disassembly.  */
-  gdb::optional<int> (*print_insn) (struct gdbarch *gdbarch,
+  std::optional<int> (*print_insn) (struct gdbarch *gdbarch,
 				    CORE_ADDR address,
 				    struct disassemble_info *info);
+
+  /* Give extension languages a chance to deal with missing debug
+     information.  OBJFILE is the file for which GDB was unable to find
+     any debug information.  */
+  ext_lang_missing_debuginfo_result
+    (*handle_missing_debuginfo) (const struct extension_language_defn *,
+				 struct objfile *objfile);
 };
 
 /* State necessary to restore a signal handler to its previous value.  */
@@ -303,8 +315,6 @@ struct active_ext_lang_state
   struct signal_handler sigint_handler;
 };
 
-extern const struct extension_language_defn *get_active_ext_lang (void);
-
 extern struct active_ext_lang_state *set_active_ext_lang
   (const struct extension_language_defn *);
 
