@@ -1,6 +1,6 @@
 /* The find command.
 
-   Copyright (C) 2008-2022 Free Software Foundation, Inc.
+   Copyright (C) 2008-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,7 +17,6 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 #include "arch-utils.h"
 #include <ctype.h>
 #include "gdbcmd.h"
@@ -30,7 +29,7 @@
 /* Copied from bfd_put_bits.  */
 
 static void
-put_bits (bfd_uint64_t data, gdb::byte_vector &buf, int bits, bfd_boolean big_p)
+put_bits (uint64_t data, gdb::byte_vector &buf, int bits, bfd_boolean big_p)
 {
   int i;
   int bytes;
@@ -121,7 +120,7 @@ parse_find_args (const char *args, ULONGEST *max_countp,
       len = value_as_long (v);
       if (len == 0)
 	{
-	  printf_filtered (_("Empty search range.\n"));
+	  gdb_printf (_("Empty search range.\n"));
 	  return pattern_buf;
 	}
       if (len < 0)
@@ -162,7 +161,7 @@ parse_find_args (const char *args, ULONGEST *max_countp,
       s = skip_spaces (s);
 
       v = parse_to_comma_and_eval (&s);
-      t = value_type (v);
+      t = v->type ();
 
       if (size != '\0')
 	{
@@ -185,9 +184,9 @@ parse_find_args (const char *args, ULONGEST *max_countp,
 	}
       else
 	{
-	  const gdb_byte *contents = value_contents (v).data ();
+	  const gdb_byte *contents = v->contents ().data ();
 	  pattern_buf.insert (pattern_buf.end (), contents,
-			      contents + TYPE_LENGTH (t));
+			      contents + t->length ());
 	}
 
       if (*s == ',')
@@ -247,7 +246,7 @@ find_command (const char *args, int from_tty)
 	break;
 
       print_address (gdbarch, found_addr, gdb_stdout);
-      printf_filtered ("\n");
+      gdb_printf ("\n");
       ++found_count;
       last_found_addr = found_addr;
 
@@ -274,10 +273,10 @@ find_command (const char *args, int from_tty)
     }
 
   if (found_count == 0)
-    printf_filtered ("Pattern not found.\n");
+    gdb_printf ("Pattern not found.\n");
   else
-    printf_filtered ("%d pattern%s found.\n", found_count,
-		     found_count > 1 ? "s" : "");
+    gdb_printf ("%d pattern%s found.\n", found_count,
+		found_count > 1 ? "s" : "");
 }
 
 void _initialize_mem_search ();
