@@ -1,6 +1,6 @@
 /* Manages interpreters for GDB, the GNU debugger.
 
-   Copyright (C) 2000-2023 Free Software Foundation, Inc.
+   Copyright (C) 2000-2024 Free Software Foundation, Inc.
 
    Written by Jim Ingham <jingham@apple.com> of Apple Computer, Inc.
 
@@ -29,7 +29,6 @@
    the readline command interface, and it is probably simpler to just let
    them take over the input in their resume proc.  */
 
-#include "defs.h"
 #include "gdbcmd.h"
 #include "ui-out.h"
 #include "gdbsupport/event-loop.h"
@@ -273,7 +272,6 @@ clear_interpreter_hooks (void)
   deprecated_print_frame_info_listing_hook = 0;
   /*print_frame_more_info_hook = 0; */
   deprecated_query_hook = 0;
-  deprecated_warning_hook = 0;
   deprecated_readline_begin_hook = 0;
   deprecated_readline_hook = 0;
   deprecated_readline_end_hook = 0;
@@ -358,15 +356,15 @@ current_interpreter (void)
 /* Helper interps_notify_* functions.  Call METHOD on the top-level interpreter
    of all UIs.  */
 
-template <typename ...Args>
+template <typename MethodType, typename ...Args>
 void
-interps_notify (void (interp::*method) (Args...), Args... args)
+interps_notify (MethodType method, Args&&... args)
 {
   SWITCH_THRU_ALL_UIS ()
     {
       interp *tli = top_level_interpreter ();
       if (tli != nullptr)
-	(tli->*method) (args...);
+	(tli->*method) (std::forward<Args> (args)...);
     }
 }
 
@@ -430,7 +428,7 @@ interps_notify_new_thread (thread_info *t)
 
 void
 interps_notify_thread_exited (thread_info *t,
-			      gdb::optional<ULONGEST> exit_code,
+			      std::optional<ULONGEST> exit_code,
 			      int silent)
 {
   interps_notify (&interp::on_thread_exited, t, exit_code, silent);
@@ -488,7 +486,7 @@ interps_notify_target_resumed (ptid_t ptid)
 /* See interps.h.  */
 
 void
-interps_notify_solib_loaded (so_list *so)
+interps_notify_solib_loaded (const solib &so)
 {
   interps_notify (&interp::on_solib_loaded, so);
 }
@@ -496,7 +494,7 @@ interps_notify_solib_loaded (so_list *so)
 /* See interps.h.  */
 
 void
-interps_notify_solib_unloaded (so_list *so)
+interps_notify_solib_unloaded (const solib &so)
 {
   interps_notify (&interp::on_solib_unloaded, so);
 }
