@@ -65,7 +65,6 @@
 #line 38 "m2-exp.y" /* yacc.c:339  */
 
 
-#include "defs.h"
 #include "expression.h"
 #include "language.h"
 #include "value.h"
@@ -100,7 +99,7 @@ static int number_sign = 1;
 
 using namespace expr;
 
-#line 104 "m2-exp.c.tmp" /* yacc.c:339  */
+#line 103 "m2-exp.c.tmp" /* yacc.c:339  */
 
 # ifndef YY_NULLPTRPTR
 #  if defined __cplusplus && 201103L <= __cplusplus
@@ -231,11 +230,11 @@ extern int yydebug;
 #define QID 304
 
 /* Value type.  */
-#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
+#if ! defined m2_exp_YYSTYPE && ! defined m2_exp_YYSTYPE_IS_DECLARED
 
-union YYSTYPE
+union m2_exp_YYSTYPE
 {
-#line 81 "m2-exp.y" /* yacc.c:355  */
+#line 80 "m2-exp.y" /* yacc.c:355  */
 
     LONGEST lval;
     ULONGEST ulval;
@@ -252,16 +251,16 @@ union YYSTYPE
     int *ivec;
   
 
-#line 256 "m2-exp.c.tmp" /* yacc.c:355  */
+#line 255 "m2-exp.c.tmp" /* yacc.c:355  */
 };
 
-typedef union YYSTYPE YYSTYPE;
-# define YYSTYPE_IS_TRIVIAL 1
-# define YYSTYPE_IS_DECLARED 1
+typedef union m2_exp_YYSTYPE m2_exp_YYSTYPE;
+# define m2_exp_YYSTYPE_IS_TRIVIAL 1
+# define m2_exp_YYSTYPE_IS_DECLARED 1
 #endif
 
 
-extern YYSTYPE yylval;
+extern m2_exp_YYSTYPE yylval;
 
 int yyparse (void);
 
@@ -269,7 +268,7 @@ int yyparse (void);
 
 /* Copy the second part of user declarations.  */
 
-#line 273 "m2-exp.c.tmp" /* yacc.c:358  */
+#line 272 "m2-exp.c.tmp" /* yacc.c:358  */
 
 #ifdef short
 # undef short
@@ -449,22 +448,22 @@ void xfree (void *); /* INFRINGES ON USER NAME SPACE */
 
 #if (! defined yyoverflow \
      && (! defined __cplusplus \
-         || (defined YYSTYPE_IS_TRIVIAL && YYSTYPE_IS_TRIVIAL)))
+         || (defined m2_exp_YYSTYPE_IS_TRIVIAL && m2_exp_YYSTYPE_IS_TRIVIAL)))
 
 /* A type that is properly aligned for any stack member.  */
-union yyalloc
+union m2_exp_yyalloc
 {
   yytype_int16 yyss_alloc;
-  YYSTYPE yyvs_alloc;
+  m2_exp_YYSTYPE yyvs_alloc;
 };
 
 /* The size of the maximum gap between one aligned stack and the next.  */
-# define YYSTACK_GAP_MAXIMUM (sizeof (union yyalloc) - 1)
+# define YYSTACK_GAP_MAXIMUM (sizeof (union m2_exp_yyalloc) - 1)
 
 /* The size of an array large to enough to hold all stacks, each with
    N elements.  */
 # define YYSTACK_BYTES(N) \
-     ((N) * (sizeof (yytype_int16) + sizeof (YYSTYPE)) \
+     ((N) * (sizeof (yytype_int16) + sizeof (m2_exp_YYSTYPE)) \
       + YYSTACK_GAP_MAXIMUM)
 
 # define YYCOPY_NEEDED 1
@@ -570,15 +569,15 @@ static const yytype_uint8 yytranslate[] =
   /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   150,   150,   151,   154,   160,   165,   164,   171,   175,
-     179,   180,   183,   187,   191,   195,   199,   203,   207,   211,
-     215,   219,   223,   227,   231,   236,   240,   249,   253,   262,
-     269,   272,   276,   280,   284,   286,   296,   292,   310,   307,
-     320,   323,   327,   332,   337,   342,   349,   356,   364,   368,
-     372,   376,   380,   384,   388,   392,   396,   398,   402,   406,
-     410,   414,   418,   422,   426,   433,   437,   441,   448,   455,
-     463,   473,   476,   484,   489,   493,   503,   515,   523,   528,
-     544,   560
+       0,   149,   149,   150,   153,   159,   164,   163,   170,   174,
+     178,   179,   182,   186,   190,   194,   198,   202,   206,   210,
+     214,   218,   222,   226,   230,   235,   239,   248,   252,   261,
+     268,   271,   275,   279,   283,   285,   295,   291,   309,   306,
+     319,   322,   326,   331,   336,   341,   348,   355,   363,   367,
+     371,   375,   379,   383,   387,   391,   395,   397,   401,   405,
+     409,   413,   417,   421,   425,   432,   436,   440,   447,   454,
+     462,   472,   475,   483,   488,   492,   502,   514,   522,   527,
+     543,   559
 };
 #endif
 
@@ -1008,7 +1007,7 @@ do {                                                                      \
 `----------------------------------------*/
 
 static void
-yy_symbol_value_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep)
+yy_symbol_value_print (FILE *yyoutput, int yytype, m2_exp_YYSTYPE const * const yyvaluep)
 {
   FILE *yyo = yyoutput;
   YYUSE (yyo);
@@ -1027,7 +1026,7 @@ yy_symbol_value_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvalue
 `--------------------------------*/
 
 static void
-yy_symbol_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep)
+yy_symbol_print (FILE *yyoutput, int yytype, m2_exp_YYSTYPE const * const yyvaluep)
 {
   YYFPRINTF (yyoutput, "%s %s (",
              yytype < YYNTOKENS ? "token" : "nterm", yytname[yytype]);
@@ -1065,7 +1064,7 @@ do {                                                            \
 `------------------------------------------------*/
 
 static void
-yy_reduce_print (yytype_int16 *yyssp, YYSTYPE *yyvsp, int yyrule)
+yy_reduce_print (yytype_int16 *yyssp, m2_exp_YYSTYPE *yyvsp, int yyrule)
 {
   unsigned long int yylno = yyrline[yyrule];
   int yynrhs = yyr2[yyrule];
@@ -1345,7 +1344,7 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
 `-----------------------------------------------*/
 
 static void
-yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep)
+yydestruct (const char *yymsg, int yytype, m2_exp_YYSTYPE *yyvaluep)
 {
   YYUSE (yyvaluep);
   if (!yymsg)
@@ -1364,7 +1363,7 @@ yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep)
 int yychar;
 
 /* The semantic value of the lookahead symbol.  */
-YYSTYPE yylval;
+m2_exp_YYSTYPE yylval;
 /* Number of syntax errors so far.  */
 int yynerrs;
 
@@ -1393,9 +1392,9 @@ yyparse (void)
     yytype_int16 *yyssp;
 
     /* The semantic value stack.  */
-    YYSTYPE yyvsa[YYINITDEPTH];
-    YYSTYPE *yyvs;
-    YYSTYPE *yyvsp;
+    m2_exp_YYSTYPE yyvsa[YYINITDEPTH];
+    m2_exp_YYSTYPE *yyvs;
+    m2_exp_YYSTYPE *yyvsp;
 
     YYSIZE_T yystacksize;
 
@@ -1405,7 +1404,7 @@ yyparse (void)
   int yytoken = 0;
   /* The variables used to return semantic value and location from the
      action routines.  */
-  YYSTYPE yyval;
+  m2_exp_YYSTYPE yyval;
 
 #if YYERROR_VERBOSE
   /* Buffer for error messages, and its allocated size.  */
@@ -1453,7 +1452,7 @@ yyparse (void)
         /* Give user a chance to xreallocate the stack.  Use copies of
            these so that the &'s don't force the real ones into
            memory.  */
-        YYSTYPE *yyvs1 = yyvs;
+        m2_exp_YYSTYPE *yyvs1 = yyvs;
         yytype_int16 *yyss1 = yyss;
 
         /* Each stack pointer address is followed by the size of the
@@ -1481,8 +1480,8 @@ yyparse (void)
 
       {
         yytype_int16 *yyss1 = yyss;
-        union yyalloc *yyptr =
-          (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
+        union m2_exp_yyalloc *yyptr =
+          (union m2_exp_yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
         if (! yyptr)
           goto yyexhaustedlab;
         YYSTACK_RELOCATE (yyss_alloc, yyss);
@@ -1609,201 +1608,201 @@ yyreduce:
   switch (yyn)
     {
         case 4:
-#line 155 "m2-exp.y" /* yacc.c:1646  */
+#line 154 "m2-exp.y" /* yacc.c:1646  */
     { pstate->push_new<type_operation> ((yyvsp[0].tval)); }
-#line 1616 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1615 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 5:
-#line 161 "m2-exp.y" /* yacc.c:1646  */
+#line 160 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap<unop_ind_operation> (); }
-#line 1622 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1621 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 6:
-#line 165 "m2-exp.y" /* yacc.c:1646  */
+#line 164 "m2-exp.y" /* yacc.c:1646  */
     { number_sign = -1; }
-#line 1628 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1627 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 7:
-#line 167 "m2-exp.y" /* yacc.c:1646  */
+#line 166 "m2-exp.y" /* yacc.c:1646  */
     { number_sign = 1;
 			  pstate->wrap<unary_neg_operation> (); }
-#line 1635 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1634 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 8:
-#line 172 "m2-exp.y" /* yacc.c:1646  */
+#line 171 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap<unary_plus_operation> (); }
-#line 1641 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1640 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 9:
-#line 176 "m2-exp.y" /* yacc.c:1646  */
+#line 175 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap<unary_logical_not_operation> (); }
-#line 1647 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1646 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 12:
-#line 184 "m2-exp.y" /* yacc.c:1646  */
+#line 183 "m2-exp.y" /* yacc.c:1646  */
     { error (_("CAP function is not implemented")); }
-#line 1653 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1652 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 13:
-#line 188 "m2-exp.y" /* yacc.c:1646  */
+#line 187 "m2-exp.y" /* yacc.c:1646  */
     { error (_("ORD function is not implemented")); }
-#line 1659 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1658 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 14:
-#line 192 "m2-exp.y" /* yacc.c:1646  */
+#line 191 "m2-exp.y" /* yacc.c:1646  */
     { error (_("ABS function is not implemented")); }
-#line 1665 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1664 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 15:
-#line 196 "m2-exp.y" /* yacc.c:1646  */
+#line 195 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap<m2_unop_high_operation> (); }
-#line 1671 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1670 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 16:
-#line 200 "m2-exp.y" /* yacc.c:1646  */
+#line 199 "m2-exp.y" /* yacc.c:1646  */
     { error (_("MIN function is not implemented")); }
-#line 1677 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1676 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 17:
-#line 204 "m2-exp.y" /* yacc.c:1646  */
+#line 203 "m2-exp.y" /* yacc.c:1646  */
     { error (_("MAX function is not implemented")); }
-#line 1683 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1682 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 18:
-#line 208 "m2-exp.y" /* yacc.c:1646  */
+#line 207 "m2-exp.y" /* yacc.c:1646  */
     { error (_("FLOAT function is not implemented")); }
-#line 1689 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1688 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 19:
-#line 212 "m2-exp.y" /* yacc.c:1646  */
+#line 211 "m2-exp.y" /* yacc.c:1646  */
     { error (_("VAL function is not implemented")); }
-#line 1695 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1694 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 20:
-#line 216 "m2-exp.y" /* yacc.c:1646  */
+#line 215 "m2-exp.y" /* yacc.c:1646  */
     { error (_("CHR function is not implemented")); }
-#line 1701 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1700 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 21:
-#line 220 "m2-exp.y" /* yacc.c:1646  */
+#line 219 "m2-exp.y" /* yacc.c:1646  */
     { error (_("ODD function is not implemented")); }
-#line 1707 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1706 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 22:
-#line 224 "m2-exp.y" /* yacc.c:1646  */
+#line 223 "m2-exp.y" /* yacc.c:1646  */
     { error (_("TRUNC function is not implemented")); }
-#line 1713 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1712 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 23:
-#line 228 "m2-exp.y" /* yacc.c:1646  */
+#line 227 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap<unop_sizeof_operation> (); }
-#line 1719 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1718 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 24:
-#line 232 "m2-exp.y" /* yacc.c:1646  */
+#line 231 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap<unop_sizeof_operation> (); }
-#line 1725 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1724 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 25:
-#line 237 "m2-exp.y" /* yacc.c:1646  */
+#line 236 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap<preinc_operation> (); }
-#line 1731 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1730 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 26:
-#line 241 "m2-exp.y" /* yacc.c:1646  */
+#line 240 "m2-exp.y" /* yacc.c:1646  */
     {
 			  operation_up rhs = pstate->pop ();
 			  operation_up lhs = pstate->pop ();
 			  pstate->push_new<assign_modify_operation>
 			    (BINOP_ADD, std::move (lhs), std::move (rhs));
 			}
-#line 1742 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1741 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 27:
-#line 250 "m2-exp.y" /* yacc.c:1646  */
+#line 249 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap<predec_operation> (); }
-#line 1748 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1747 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 28:
-#line 254 "m2-exp.y" /* yacc.c:1646  */
+#line 253 "m2-exp.y" /* yacc.c:1646  */
     {
 			  operation_up rhs = pstate->pop ();
 			  operation_up lhs = pstate->pop ();
 			  pstate->push_new<assign_modify_operation>
 			    (BINOP_SUB, std::move (lhs), std::move (rhs));
 			}
-#line 1759 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1758 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 29:
-#line 263 "m2-exp.y" /* yacc.c:1646  */
+#line 262 "m2-exp.y" /* yacc.c:1646  */
     {
 			  pstate->push_new<structop_operation>
 			    (pstate->pop (), copy_name ((yyvsp[0].sval)));
 			}
-#line 1768 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1767 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 31:
-#line 273 "m2-exp.y" /* yacc.c:1646  */
+#line 272 "m2-exp.y" /* yacc.c:1646  */
     { error (_("Sets are not implemented."));}
-#line 1774 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1773 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 32:
-#line 277 "m2-exp.y" /* yacc.c:1646  */
+#line 276 "m2-exp.y" /* yacc.c:1646  */
     { error (_("Sets are not implemented."));}
-#line 1780 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1779 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 33:
-#line 281 "m2-exp.y" /* yacc.c:1646  */
+#line 280 "m2-exp.y" /* yacc.c:1646  */
     { error (_("Sets are not implemented."));}
-#line 1786 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1785 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 34:
-#line 285 "m2-exp.y" /* yacc.c:1646  */
+#line 284 "m2-exp.y" /* yacc.c:1646  */
     { error (_("Sets are not implemented."));}
-#line 1792 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1791 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 35:
-#line 287 "m2-exp.y" /* yacc.c:1646  */
+#line 286 "m2-exp.y" /* yacc.c:1646  */
     { error (_("Sets are not implemented."));}
-#line 1798 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1797 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 36:
-#line 296 "m2-exp.y" /* yacc.c:1646  */
+#line 295 "m2-exp.y" /* yacc.c:1646  */
     { pstate->start_arglist(); }
-#line 1804 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1803 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 37:
-#line 298 "m2-exp.y" /* yacc.c:1646  */
+#line 297 "m2-exp.y" /* yacc.c:1646  */
     {
 			  gdb_assert (pstate->arglist_len > 0);
 			  std::vector<operation_up> args
@@ -1811,217 +1810,217 @@ yyreduce:
 			  pstate->push_new<multi_subscript_operation>
 			    (pstate->pop (), std::move (args));
 			}
-#line 1816 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1815 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 38:
-#line 310 "m2-exp.y" /* yacc.c:1646  */
+#line 309 "m2-exp.y" /* yacc.c:1646  */
     { pstate->start_arglist (); }
-#line 1822 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1821 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 39:
-#line 312 "m2-exp.y" /* yacc.c:1646  */
+#line 311 "m2-exp.y" /* yacc.c:1646  */
     {
 			  std::vector<operation_up> args
 			    = pstate->pop_vector (pstate->end_arglist ());
 			  pstate->push_new<funcall_operation>
 			    (pstate->pop (), std::move (args));
 			}
-#line 1833 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1832 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 41:
-#line 324 "m2-exp.y" /* yacc.c:1646  */
+#line 323 "m2-exp.y" /* yacc.c:1646  */
     { pstate->arglist_len = 1; }
-#line 1839 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1838 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 42:
-#line 328 "m2-exp.y" /* yacc.c:1646  */
+#line 327 "m2-exp.y" /* yacc.c:1646  */
     { pstate->arglist_len++; }
-#line 1845 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1844 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 43:
-#line 333 "m2-exp.y" /* yacc.c:1646  */
+#line 332 "m2-exp.y" /* yacc.c:1646  */
     { pstate->arglist_len = 1; }
-#line 1851 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1850 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 44:
-#line 338 "m2-exp.y" /* yacc.c:1646  */
+#line 337 "m2-exp.y" /* yacc.c:1646  */
     { pstate->arglist_len++; }
-#line 1857 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1856 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 45:
-#line 343 "m2-exp.y" /* yacc.c:1646  */
+#line 342 "m2-exp.y" /* yacc.c:1646  */
     {
 			  pstate->push_new<unop_memval_operation>
 			    (pstate->pop (), (yyvsp[-2].tval));
 			}
-#line 1866 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1865 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 46:
-#line 350 "m2-exp.y" /* yacc.c:1646  */
+#line 349 "m2-exp.y" /* yacc.c:1646  */
     {
 			  pstate->push_new<unop_cast_operation>
 			    (pstate->pop (), (yyvsp[-3].tval));
 			}
-#line 1875 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1874 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 47:
-#line 357 "m2-exp.y" /* yacc.c:1646  */
+#line 356 "m2-exp.y" /* yacc.c:1646  */
     { }
-#line 1881 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1880 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 48:
-#line 365 "m2-exp.y" /* yacc.c:1646  */
+#line 364 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<repeat_operation> (); }
-#line 1887 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1886 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 49:
-#line 369 "m2-exp.y" /* yacc.c:1646  */
+#line 368 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<mul_operation> (); }
-#line 1893 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1892 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 50:
-#line 373 "m2-exp.y" /* yacc.c:1646  */
+#line 372 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<div_operation> (); }
-#line 1899 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1898 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 51:
-#line 377 "m2-exp.y" /* yacc.c:1646  */
+#line 376 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<intdiv_operation> (); }
-#line 1905 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1904 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 52:
-#line 381 "m2-exp.y" /* yacc.c:1646  */
+#line 380 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<rem_operation> (); }
-#line 1911 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1910 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 53:
-#line 385 "m2-exp.y" /* yacc.c:1646  */
+#line 384 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<add_operation> (); }
-#line 1917 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1916 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 54:
-#line 389 "m2-exp.y" /* yacc.c:1646  */
+#line 388 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<sub_operation> (); }
-#line 1923 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1922 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 55:
-#line 393 "m2-exp.y" /* yacc.c:1646  */
+#line 392 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<equal_operation> (); }
-#line 1929 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1928 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 56:
-#line 397 "m2-exp.y" /* yacc.c:1646  */
+#line 396 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<notequal_operation> (); }
-#line 1935 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1934 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 57:
-#line 399 "m2-exp.y" /* yacc.c:1646  */
+#line 398 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<notequal_operation> (); }
-#line 1941 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1940 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 58:
-#line 403 "m2-exp.y" /* yacc.c:1646  */
+#line 402 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<leq_operation> (); }
-#line 1947 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1946 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 59:
-#line 407 "m2-exp.y" /* yacc.c:1646  */
+#line 406 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<geq_operation> (); }
-#line 1953 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1952 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 60:
-#line 411 "m2-exp.y" /* yacc.c:1646  */
+#line 410 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<less_operation> (); }
-#line 1959 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1958 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 61:
-#line 415 "m2-exp.y" /* yacc.c:1646  */
+#line 414 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<gtr_operation> (); }
-#line 1965 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1964 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 62:
-#line 419 "m2-exp.y" /* yacc.c:1646  */
+#line 418 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<logical_and_operation> (); }
-#line 1971 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1970 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 63:
-#line 423 "m2-exp.y" /* yacc.c:1646  */
+#line 422 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<logical_or_operation> (); }
-#line 1977 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1976 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 64:
-#line 427 "m2-exp.y" /* yacc.c:1646  */
+#line 426 "m2-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<assign_operation> (); }
-#line 1983 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1982 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 65:
-#line 434 "m2-exp.y" /* yacc.c:1646  */
+#line 433 "m2-exp.y" /* yacc.c:1646  */
     { pstate->push_new<bool_operation> ((yyvsp[0].ulval)); }
-#line 1989 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1988 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 66:
-#line 438 "m2-exp.y" /* yacc.c:1646  */
+#line 437 "m2-exp.y" /* yacc.c:1646  */
     { pstate->push_new<bool_operation> ((yyvsp[0].ulval)); }
-#line 1995 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 1994 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 67:
-#line 442 "m2-exp.y" /* yacc.c:1646  */
+#line 441 "m2-exp.y" /* yacc.c:1646  */
     {
 			  pstate->push_new<long_const_operation>
 			    (parse_m2_type (pstate)->builtin_int, (yyvsp[0].lval));
 			}
-#line 2004 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 2003 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 68:
-#line 449 "m2-exp.y" /* yacc.c:1646  */
+#line 448 "m2-exp.y" /* yacc.c:1646  */
     {
 			  pstate->push_new<long_const_operation>
 			    (parse_m2_type (pstate)->builtin_card, (yyvsp[0].ulval));
 			}
-#line 2013 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 2012 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 69:
-#line 456 "m2-exp.y" /* yacc.c:1646  */
+#line 455 "m2-exp.y" /* yacc.c:1646  */
     {
 			  pstate->push_new<long_const_operation>
 			    (parse_m2_type (pstate)->builtin_char, (yyvsp[0].ulval));
 			}
-#line 2022 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 2021 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 70:
-#line 464 "m2-exp.y" /* yacc.c:1646  */
+#line 463 "m2-exp.y" /* yacc.c:1646  */
     {
 			  float_data data;
 			  std::copy (std::begin ((yyvsp[0].val)), std::end ((yyvsp[0].val)),
@@ -2029,74 +2028,74 @@ yyreduce:
 			  pstate->push_new<float_const_operation>
 			    (parse_m2_type (pstate)->builtin_real, data);
 			}
-#line 2034 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 2033 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 72:
-#line 477 "m2-exp.y" /* yacc.c:1646  */
+#line 476 "m2-exp.y" /* yacc.c:1646  */
     {
 			  pstate->push_new<long_const_operation>
 			    (parse_m2_type (pstate)->builtin_int,
 			     (yyvsp[-1].tval)->length ());
 			}
-#line 2044 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 2043 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 73:
-#line 485 "m2-exp.y" /* yacc.c:1646  */
+#line 484 "m2-exp.y" /* yacc.c:1646  */
     { error (_("strings are not implemented")); }
-#line 2050 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 2049 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 74:
-#line 490 "m2-exp.y" /* yacc.c:1646  */
+#line 489 "m2-exp.y" /* yacc.c:1646  */
     { (yyval.bval) = (yyvsp[0].sym)->value_block (); }
-#line 2056 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 2055 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 75:
-#line 494 "m2-exp.y" /* yacc.c:1646  */
+#line 493 "m2-exp.y" /* yacc.c:1646  */
     { struct symbol *sym
 			    = lookup_symbol (copy_name ((yyvsp[0].sval)).c_str (),
 					     pstate->expression_context_block,
-					     VAR_DOMAIN, 0).symbol;
+					     SEARCH_VFT, 0).symbol;
 			  (yyval.sym) = sym;}
-#line 2066 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 2065 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 76:
-#line 504 "m2-exp.y" /* yacc.c:1646  */
+#line 503 "m2-exp.y" /* yacc.c:1646  */
     { struct symbol *tem
 			    = lookup_symbol (copy_name ((yyvsp[0].sval)).c_str (), (yyvsp[-2].bval),
-					     VAR_DOMAIN, 0).symbol;
+					     SEARCH_VFT, 0).symbol;
 			  if (!tem || tem->aclass () != LOC_BLOCK)
 			    error (_("No function \"%s\" in specified context."),
 				   copy_name ((yyvsp[0].sval)).c_str ());
 			  (yyval.sym) = tem;
 			}
-#line 2079 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 2078 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 77:
-#line 516 "m2-exp.y" /* yacc.c:1646  */
+#line 515 "m2-exp.y" /* yacc.c:1646  */
     {
 			  block_symbol sym { (yyvsp[0].sym), nullptr };
 			  pstate->push_new<var_value_operation> (sym);
 			}
-#line 2088 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 2087 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 78:
-#line 524 "m2-exp.y" /* yacc.c:1646  */
+#line 523 "m2-exp.y" /* yacc.c:1646  */
     { pstate->push_dollar ((yyvsp[0].sval)); }
-#line 2094 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 2093 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 79:
-#line 529 "m2-exp.y" /* yacc.c:1646  */
+#line 528 "m2-exp.y" /* yacc.c:1646  */
     { struct block_symbol sym
 			    = lookup_symbol (copy_name ((yyvsp[0].sval)).c_str (), (yyvsp[-2].bval),
-					     VAR_DOMAIN, 0);
+					     SEARCH_VFT, 0);
 
 			  if (sym.symbol == 0)
 			    error (_("No symbol \"%s\" in specified context."),
@@ -2106,11 +2105,11 @@ yyreduce:
 
 			  pstate->push_new<var_value_operation> (sym);
 			}
-#line 2111 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 2110 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 80:
-#line 545 "m2-exp.y" /* yacc.c:1646  */
+#line 544 "m2-exp.y" /* yacc.c:1646  */
     { struct block_symbol sym;
 			  struct field_of_this_result is_a_field_of_this;
 
@@ -2118,27 +2117,27 @@ yyreduce:
 			  sym
 			    = lookup_symbol (name.c_str (),
 					     pstate->expression_context_block,
-					     VAR_DOMAIN,
+					     SEARCH_VFT,
 					     &is_a_field_of_this);
 
 			  pstate->push_symbol (name.c_str (), sym);
 			}
-#line 2128 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 2127 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 81:
-#line 561 "m2-exp.y" /* yacc.c:1646  */
+#line 560 "m2-exp.y" /* yacc.c:1646  */
     { (yyval.tval)
 			    = lookup_typename (pstate->language (),
 					       copy_name ((yyvsp[0].sval)).c_str (),
 					       pstate->expression_context_block,
 					       0);
 			}
-#line 2139 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 2138 "m2-exp.c.tmp" /* yacc.c:1646  */
     break;
 
 
-#line 2143 "m2-exp.c.tmp" /* yacc.c:1646  */
+#line 2142 "m2-exp.c.tmp" /* yacc.c:1646  */
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -2366,7 +2365,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 570 "m2-exp.y" /* yacc.c:1906  */
+#line 569 "m2-exp.y" /* yacc.c:1906  */
 
 
 /* Take care of parsing a number (anything that starts with a digit).
@@ -2727,7 +2726,7 @@ yylex (void)
     if (lookup_symtab (tmp.c_str ()))
       return BLOCKNAME;
     sym = lookup_symbol (tmp.c_str (), pstate->expression_context_block,
-			 VAR_DOMAIN, 0).symbol;
+			 SEARCH_VFT, 0).symbol;
     if (sym && sym->aclass () == LOC_BLOCK)
       return BLOCKNAME;
     if (lookup_typename (pstate->language (),
@@ -2806,8 +2805,5 @@ m2_language::parser (struct parser_state *par_state) const
 static void
 yyerror (const char *msg)
 {
-  if (pstate->prev_lexptr)
-    pstate->lexptr = pstate->prev_lexptr;
-
-  error (_("A %s in expression, near `%s'."), msg, pstate->lexptr);
+  pstate->parse_error (msg);
 }
