@@ -1,5 +1,5 @@
 /* C preprocessor macro expansion commands for GDB.
-   Copyright (C) 2002-2022 Free Software Foundation, Inc.
+   Copyright (C) 2002-2024 Free Software Foundation, Inc.
    Contributed by Red Hat, Inc.
 
    This file is part of GDB.
@@ -18,7 +18,6 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 
-#include "defs.h"
 #include "macrotab.h"
 #include "macroexp.h"
 #include "macroscope.h"
@@ -41,7 +40,7 @@ static struct cmd_list_element *macrolist;
 static void
 macro_inform_no_debuginfo (void)
 {
-  puts_filtered ("GDB has no preprocessor macro information for that code.\n");
+  gdb_puts ("GDB has no preprocessor macro information for that code.\n");
 }
 
 static void
@@ -63,9 +62,9 @@ macro_expand_command (const char *exp, int from_tty)
     {
       gdb::unique_xmalloc_ptr<char> expanded = macro_expand (exp, *ms);
 
-      puts_filtered ("expands to: ");
-      puts_filtered (expanded.get ());
-      puts_filtered ("\n");
+      gdb_puts ("expands to: ");
+      gdb_puts (expanded.get ());
+      gdb_puts ("\n");
     }
   else
     macro_inform_no_debuginfo ();
@@ -91,9 +90,9 @@ macro_expand_once_command (const char *exp, int from_tty)
     {
       gdb::unique_xmalloc_ptr<char> expanded = macro_expand_once (exp, *ms);
 
-      puts_filtered ("expands to: ");
-      puts_filtered (expanded.get ());
-      puts_filtered ("\n");
+      gdb_puts ("expands to: ");
+      gdb_puts (expanded.get ());
+      gdb_puts ("\n");
     }
   else
     macro_inform_no_debuginfo ();
@@ -109,17 +108,17 @@ show_pp_source_pos (struct ui_file *stream,
 		    int line)
 {
   std::string fullname = macro_source_fullname (file);
-  fprintf_filtered (stream, "%ps:%d\n",
-		    styled_string (file_name_style.style (),
-				   fullname.c_str ()),
-		    line);
+  gdb_printf (stream, "%ps:%d\n",
+	      styled_string (file_name_style.style (),
+			     fullname.c_str ()),
+	      line);
 
   while (file->included_by)
     {
       fullname = macro_source_fullname (file->included_by);
-      fputs_filtered (_("  included at "), stream);
+      gdb_puts (_("  included at "), stream);
       fputs_styled (fullname.c_str (), file_name_style.style (), stream);
-      fprintf_filtered (stream, ":%d\n", file->included_at_line);
+      gdb_printf (stream, ":%d\n", file->included_at_line);
       file = file->included_by;
     }
 }
@@ -137,32 +136,32 @@ print_macro_definition (const char *name,
 			struct macro_source_file *file,
 			int line)
 {
-  printf_filtered ("Defined at ");
+  gdb_printf ("Defined at ");
   show_pp_source_pos (gdb_stdout, file, line);
 
   if (line != 0)
-    printf_filtered ("#define %s", name);
+    gdb_printf ("#define %s", name);
   else
-    printf_filtered ("-D%s", name);
+    gdb_printf ("-D%s", name);
 
   if (d->kind == macro_function_like)
     {
       int i;
 
-      puts_filtered ("(");
+      gdb_puts ("(");
       for (i = 0; i < d->argc; i++)
 	{
-	  puts_filtered (d->argv[i]);
+	  gdb_puts (d->argv[i]);
 	  if (i + 1 < d->argc)
-	    puts_filtered (", ");
+	    gdb_puts (", ");
 	}
-      puts_filtered (")");
+      gdb_puts (")");
     }
 
   if (line != 0)
-    printf_filtered (" %s\n", d->replacement);
+    gdb_printf (" %s\n", d->replacement);
   else
-    printf_filtered ("=%s\n", d->replacement);
+    gdb_printf ("=%s\n", d->replacement);
 }
 
 /* The implementation of the `info macro' command.  */
@@ -229,9 +228,9 @@ info_macro_command (const char *args, int from_tty)
 	}
       else
 	{
-	  printf_filtered ("The symbol `%s' has no definition as a C/C++"
-			   " preprocessor macro\n"
-			   "at ", name);
+	  gdb_printf ("The symbol `%s' has no definition as a C/C++"
+		      " preprocessor macro\n"
+		      "at ", name);
 	  show_pp_source_pos (gdb_stdout, ms->file, ms->line);
 	}
     }
@@ -423,18 +422,18 @@ static void
 print_one_macro (const char *name, const struct macro_definition *macro,
 		 struct macro_source_file *source, int line)
 {
-  printf_filtered ("macro define %s", name);
+  gdb_printf ("macro define %s", name);
   if (macro->kind == macro_function_like)
     {
       int i;
 
-      printf_filtered ("(");
+      gdb_printf ("(");
       for (i = 0; i < macro->argc; ++i)
-	printf_filtered ("%s%s", (i > 0) ? ", " : "",
+	gdb_printf ("%s%s", (i > 0) ? ", " : "",
 			 macro->argv[i]);
-      printf_filtered (")");
+      gdb_printf (")");
     }
-  printf_filtered (" %s\n", macro->replacement);
+  gdb_printf (" %s\n", macro->replacement);
 }
 
 
