@@ -1,6 +1,6 @@
 /* Maintenance commands for testing the settings framework.
 
-   Copyright (C) 2019-2022 Free Software Foundation, Inc.
+   Copyright (C) 2019-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -18,7 +18,6 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 
-#include "defs.h"
 #include "gdbcmd.h"
 
 /* Command list for "maint set test-settings".  */
@@ -75,7 +74,7 @@ maintenance_show_test_settings_value_cmd
   (struct ui_file *file, int from_tty,
    struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, (("%s\n")), value);
+  gdb_printf (file, (("%s\n")), value);
 }
 
 
