@@ -1,6 +1,6 @@
 /* Memory attributes support, for GDB.
 
-   Copyright (C) 2001-2022 Free Software Foundation, Inc.
+   Copyright (C) 2001-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,7 +17,6 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 #include "command.h"
 #include "gdbcmd.h"
 #include "memattr.h"
@@ -29,6 +28,8 @@
 #include "cli/cli-utils.h"
 #include <algorithm>
 #include "gdbarch.h"
+#include "inferior.h"
+#include "progspace.h"
 
 static std::vector<mem_region> user_mem_region_list, target_mem_region_list;
 static std::vector<mem_region> *mem_region_list = &target_mem_region_list;
@@ -354,10 +355,10 @@ info_mem_command (const char *args, int from_tty)
   gdb_printf ("Num ");
   gdb_printf ("Enb ");
   gdb_printf ("Low Addr   ");
-  if (gdbarch_addr_bit (target_gdbarch ()) > 32)
+  if (gdbarch_addr_bit (current_inferior ()->arch ()) > 32)
     gdb_printf ("        ");
   gdb_printf ("High Addr  ");
-  if (gdbarch_addr_bit (target_gdbarch ()) > 32)
+  if (gdbarch_addr_bit (current_inferior ()->arch ()) > 32)
     gdb_printf ("        ");
   gdb_printf ("Attrs ");
   gdb_printf ("\n");
@@ -369,14 +370,14 @@ info_mem_command (const char *args, int from_tty)
       gdb_printf ("%-3d %-3c\t",
 		  m.number,
 		  m.enabled_p ? 'y' : 'n');
-      if (gdbarch_addr_bit (target_gdbarch ()) <= 32)
+      if (gdbarch_addr_bit (current_inferior ()->arch ()) <= 32)
 	tmp = hex_string_custom (m.lo, 8);
       else
 	tmp = hex_string_custom (m.lo, 16);
       
       gdb_printf ("%s ", tmp);
 
-      if (gdbarch_addr_bit (target_gdbarch ()) <= 32)
+      if (gdbarch_addr_bit (current_inferior ()->arch ()) <= 32)
 	{
 	  if (m.hi == 0)
 	    tmp = "0x100000000";
@@ -482,7 +483,7 @@ enable_mem_command (const char *args, int from_tty)
 {
   require_user_regions (from_tty);
 
-  target_dcache_invalidate ();
+  target_dcache_invalidate (current_program_space->aspace);
 
   if (args == NULL || *args == '\0')
     { /* Enable all mem regions.  */
@@ -520,7 +521,7 @@ disable_mem_command (const char *args, int from_tty)
 {
   require_user_regions (from_tty);
 
-  target_dcache_invalidate ();
+  target_dcache_invalidate (current_program_space->aspace);
 
   if (args == NULL || *args == '\0')
     {
@@ -566,7 +567,7 @@ delete_mem_command (const char *args, int from_tty)
 {
   require_user_regions (from_tty);
 
-  target_dcache_invalidate ();
+  target_dcache_invalidate (current_program_space->aspace);
 
   if (args == NULL || *args == '\0')
     {
