@@ -1,6 +1,6 @@
 /* Native-dependent code for GNU/Linux on MIPS processors.
 
-   Copyright (C) 2001-2023 Free Software Foundation, Inc.
+   Copyright (C) 2001-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,7 +17,6 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 #include "command.h"
 #include "gdbcmd.h"
 #include "inferior.h"
@@ -516,7 +515,7 @@ mips_show_dr (const char *func, CORE_ADDR addr,
   if (addr || len)
     gdb_printf (gdb_stdlog,
 		" (addr=%s, len=%d, type=%s)",
-		paddress (target_gdbarch (), addr), len,
+		paddress (current_inferior ()->arch (), addr), len,
 		type == hw_write ? "data-write"
 		: (type == hw_read ? "data-read"
 		   : (type == hw_access ? "data-read/write"
@@ -526,10 +525,10 @@ mips_show_dr (const char *func, CORE_ADDR addr,
 
   for (i = 0; i < MAX_DEBUG_REGISTER; i++)
     gdb_printf (gdb_stdlog, "\tDR%d: lo=%s, hi=%s\n", i,
-		paddress (target_gdbarch (),
+		paddress (current_inferior ()->arch (),
 			  mips_linux_watch_get_watchlo (&watch_mirror,
 							i)),
-		paddress (target_gdbarch (),
+		paddress (current_inferior ()->arch (),
 			  mips_linux_watch_get_watchhi (&watch_mirror,
 							i)));
 }
