@@ -1,5 +1,5 @@
 /* Target-dependent header for the Nios II architecture, for GDB.
-   Copyright (C) 2012-2022 Free Software Foundation, Inc.
+   Copyright (C) 2012-2024 Free Software Foundation, Inc.
    Contributed by Mentor Graphics, Inc.
 
    This file is part of GDB.
@@ -20,6 +20,8 @@
 #ifndef NIOS2_TDEP_H
 #define NIOS2_TDEP_H
 
+#include "gdbarch.h"
+
 /* Nios II ISA specific encodings and macros.  */
 #include "opcode/nios2.h"
 
@@ -67,22 +69,22 @@
 #define NIOS2_CDX_OPCODE_SIZE 2
 
 /* Target-dependent structure in gdbarch.  */
-struct gdbarch_tdep
+struct nios2_gdbarch_tdep : gdbarch_tdep_base
 {
   /* Assumes FRAME is stopped at a syscall (trap) instruction; returns
      the expected next PC.  */
-  CORE_ADDR (*syscall_next_pc) (struct frame_info *frame,
-				const struct nios2_opcode *op);
+  CORE_ADDR (*syscall_next_pc) (const frame_info_ptr &frame,
+				const struct nios2_opcode *op) = nullptr;
 
   /* Returns true if PC points to a kernel helper function.  */
-  bool (*is_kernel_helper) (CORE_ADDR pc);
+  bool (*is_kernel_helper) (CORE_ADDR pc) = nullptr;
 
   /* Offset to PC value in jump buffer.
      If this is negative, longjmp support will be disabled.  */
-  int jb_pc;
+  int jb_pc = 0;
 };
 
-extern struct target_desc *tdesc_nios2_linux;
-extern struct target_desc *tdesc_nios2;
+extern const struct target_desc *tdesc_nios2_linux;
+extern const struct target_desc *tdesc_nios2;
 
 #endif /* NIOS2_TDEP_H */
