@@ -65,7 +65,6 @@
 #line 44 "p-exp.y" /* yacc.c:339  */
 
 
-#include "defs.h"
 #include <ctype.h>
 #include "expression.h"
 #include "value.h"
@@ -98,9 +97,11 @@ static void yyerror (const char *);
 
 static char *uptok (const char *, int);
 
+static const char *pascal_skip_string (const char *str);
+
 using namespace expr;
 
-#line 104 "p-exp.c.tmp" /* yacc.c:339  */
+#line 105 "p-exp.c.tmp" /* yacc.c:339  */
 
 # ifndef YY_NULLPTRPTR
 #  if defined __cplusplus && 201103L <= __cplusplus
@@ -207,11 +208,11 @@ extern int yydebug;
 #define BLOCKNAME 292
 
 /* Value type.  */
-#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
+#if ! defined p_exp_YYSTYPE && ! defined p_exp_YYSTYPE_IS_DECLARED
 
-union YYSTYPE
+union p_exp_YYSTYPE
 {
-#line 87 "p-exp.y" /* yacc.c:355  */
+#line 88 "p-exp.y" /* yacc.c:355  */
 
     LONGEST lval;
     struct {
@@ -236,27 +237,27 @@ union YYSTYPE
     int *ivec;
   
 
-#line 240 "p-exp.c.tmp" /* yacc.c:355  */
+#line 241 "p-exp.c.tmp" /* yacc.c:355  */
 };
 
-typedef union YYSTYPE YYSTYPE;
-# define YYSTYPE_IS_TRIVIAL 1
-# define YYSTYPE_IS_DECLARED 1
+typedef union p_exp_YYSTYPE p_exp_YYSTYPE;
+# define p_exp_YYSTYPE_IS_TRIVIAL 1
+# define p_exp_YYSTYPE_IS_DECLARED 1
 #endif
 
 
-extern YYSTYPE yylval;
+extern p_exp_YYSTYPE yylval;
 
 int yyparse (void);
 
 
 
 /* Copy the second part of user declarations.  */
-#line 111 "p-exp.y" /* yacc.c:358  */
+#line 112 "p-exp.y" /* yacc.c:358  */
 
-/* YYSTYPE gets defined by %union */
+/* p_exp_YYSTYPE gets defined by %union */
 static int parse_number (struct parser_state *,
-			 const char *, int, int, YYSTYPE *);
+			 const char *, int, int, p_exp_YYSTYPE *);
 
 static struct type *current_type;
 static int leftdiv_is_integer;
@@ -264,7 +265,7 @@ static void push_current_type (void);
 static void pop_current_type (void);
 static int search_field;
 
-#line 268 "p-exp.c.tmp" /* yacc.c:358  */
+#line 269 "p-exp.c.tmp" /* yacc.c:358  */
 
 #ifdef short
 # undef short
@@ -444,22 +445,22 @@ void xfree (void *); /* INFRINGES ON USER NAME SPACE */
 
 #if (! defined yyoverflow \
      && (! defined __cplusplus \
-         || (defined YYSTYPE_IS_TRIVIAL && YYSTYPE_IS_TRIVIAL)))
+         || (defined p_exp_YYSTYPE_IS_TRIVIAL && p_exp_YYSTYPE_IS_TRIVIAL)))
 
 /* A type that is properly aligned for any stack member.  */
-union yyalloc
+union p_exp_yyalloc
 {
   yytype_int16 yyss_alloc;
-  YYSTYPE yyvs_alloc;
+  p_exp_YYSTYPE yyvs_alloc;
 };
 
 /* The size of the maximum gap between one aligned stack and the next.  */
-# define YYSTACK_GAP_MAXIMUM (sizeof (union yyalloc) - 1)
+# define YYSTACK_GAP_MAXIMUM (sizeof (union p_exp_yyalloc) - 1)
 
 /* The size of an array large to enough to hold all stacks, each with
    N elements.  */
 # define YYSTACK_BYTES(N) \
-     ((N) * (sizeof (yytype_int16) + sizeof (YYSTYPE)) \
+     ((N) * (sizeof (yytype_int16) + sizeof (p_exp_YYSTYPE)) \
       + YYSTACK_GAP_MAXIMUM)
 
 # define YYCOPY_NEEDED 1
@@ -564,14 +565,14 @@ static const yytype_uint8 yytranslate[] =
   /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   192,   192,   192,   200,   201,   204,   210,   211,   216,
-     222,   228,   232,   236,   240,   245,   249,   267,   283,   292,
-     303,   301,   326,   323,   340,   341,   343,   347,   361,   367,
-     371,   371,   392,   396,   400,   404,   408,   412,   416,   423,
-     430,   437,   444,   451,   458,   462,   466,   470,   474,   481,
-     488,   496,   508,   517,   520,   546,   555,   559,   581,   608,
-     627,   640,   654,   668,   669,   680,   738,   749,   753,   755,
-     757,   762,   772,   773,   774,   775,   778,   779
+       0,   193,   193,   193,   201,   202,   205,   211,   212,   217,
+     223,   229,   233,   237,   241,   246,   250,   268,   284,   293,
+     304,   302,   327,   324,   341,   342,   344,   348,   362,   368,
+     372,   372,   393,   397,   401,   405,   409,   413,   417,   424,
+     431,   438,   445,   452,   459,   463,   467,   471,   475,   482,
+     489,   497,   509,   518,   521,   547,   556,   560,   582,   609,
+     628,   642,   656,   670,   671,   682,   740,   751,   755,   757,
+     759,   764,   774,   775,   776,   777,   780,   781
 };
 #endif
 
@@ -876,7 +877,7 @@ do {                                                                      \
 `----------------------------------------*/
 
 static void
-yy_symbol_value_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep)
+yy_symbol_value_print (FILE *yyoutput, int yytype, p_exp_YYSTYPE const * const yyvaluep)
 {
   FILE *yyo = yyoutput;
   YYUSE (yyo);
@@ -895,7 +896,7 @@ yy_symbol_value_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvalue
 `--------------------------------*/
 
 static void
-yy_symbol_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep)
+yy_symbol_print (FILE *yyoutput, int yytype, p_exp_YYSTYPE const * const yyvaluep)
 {
   YYFPRINTF (yyoutput, "%s %s (",
              yytype < YYNTOKENS ? "token" : "nterm", yytname[yytype]);
@@ -933,7 +934,7 @@ do {                                                            \
 `------------------------------------------------*/
 
 static void
-yy_reduce_print (yytype_int16 *yyssp, YYSTYPE *yyvsp, int yyrule)
+yy_reduce_print (yytype_int16 *yyssp, p_exp_YYSTYPE *yyvsp, int yyrule)
 {
   unsigned long int yylno = yyrline[yyrule];
   int yynrhs = yyr2[yyrule];
@@ -1213,7 +1214,7 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
 `-----------------------------------------------*/
 
 static void
-yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep)
+yydestruct (const char *yymsg, int yytype, p_exp_YYSTYPE *yyvaluep)
 {
   YYUSE (yyvaluep);
   if (!yymsg)
@@ -1232,7 +1233,7 @@ yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep)
 int yychar;
 
 /* The semantic value of the lookahead symbol.  */
-YYSTYPE yylval;
+p_exp_YYSTYPE yylval;
 /* Number of syntax errors so far.  */
 int yynerrs;
 
@@ -1261,9 +1262,9 @@ yyparse (void)
     yytype_int16 *yyssp;
 
     /* The semantic value stack.  */
-    YYSTYPE yyvsa[YYINITDEPTH];
-    YYSTYPE *yyvs;
-    YYSTYPE *yyvsp;
+    p_exp_YYSTYPE yyvsa[YYINITDEPTH];
+    p_exp_YYSTYPE *yyvs;
+    p_exp_YYSTYPE *yyvsp;
 
     YYSIZE_T yystacksize;
 
@@ -1273,7 +1274,7 @@ yyparse (void)
   int yytoken = 0;
   /* The variables used to return semantic value and location from the
      action routines.  */
-  YYSTYPE yyval;
+  p_exp_YYSTYPE yyval;
 
 #if YYERROR_VERBOSE
   /* Buffer for error messages, and its allocated size.  */
@@ -1321,7 +1322,7 @@ yyparse (void)
         /* Give user a chance to xreallocate the stack.  Use copies of
            these so that the &'s don't force the real ones into
            memory.  */
-        YYSTYPE *yyvs1 = yyvs;
+        p_exp_YYSTYPE *yyvs1 = yyvs;
         yytype_int16 *yyss1 = yyss;
 
         /* Each stack pointer address is followed by the size of the
@@ -1349,8 +1350,8 @@ yyparse (void)
 
       {
         yytype_int16 *yyss1 = yyss;
-        union yyalloc *yyptr =
-          (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
+        union p_exp_yyalloc *yyptr =
+          (union p_exp_yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
         if (! yyptr)
           goto yyexhaustedlab;
         YYSTACK_RELOCATE (yyss_alloc, yyss);
@@ -1477,82 +1478,82 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 192 "p-exp.y" /* yacc.c:1646  */
+#line 193 "p-exp.y" /* yacc.c:1646  */
     { current_type = NULL;
 		  search_field = 0;
 		  leftdiv_is_integer = 0;
 		}
-#line 1487 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1488 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 3:
-#line 196 "p-exp.y" /* yacc.c:1646  */
+#line 197 "p-exp.y" /* yacc.c:1646  */
     {}
-#line 1493 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1494 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 6:
-#line 205 "p-exp.y" /* yacc.c:1646  */
+#line 206 "p-exp.y" /* yacc.c:1646  */
     {
 			  pstate->push_new<type_operation> ((yyvsp[0].tval));
 			  current_type = (yyvsp[0].tval); }
-#line 1501 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1502 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 8:
-#line 212 "p-exp.y" /* yacc.c:1646  */
+#line 213 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<comma_operation> (); }
-#line 1507 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1508 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 9:
-#line 217 "p-exp.y" /* yacc.c:1646  */
+#line 218 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap<unop_ind_operation> ();
 			  if (current_type)
 			    current_type = current_type->target_type (); }
-#line 1515 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1516 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 10:
-#line 223 "p-exp.y" /* yacc.c:1646  */
+#line 224 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap<unop_addr_operation> ();
 			  if (current_type)
 			    current_type = TYPE_POINTER_TYPE (current_type); }
-#line 1523 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1524 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 11:
-#line 229 "p-exp.y" /* yacc.c:1646  */
+#line 230 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap<unary_neg_operation> (); }
-#line 1529 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1530 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 12:
-#line 233 "p-exp.y" /* yacc.c:1646  */
+#line 234 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap<unary_logical_not_operation> (); }
-#line 1535 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1536 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 13:
-#line 237 "p-exp.y" /* yacc.c:1646  */
+#line 238 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap<preinc_operation> (); }
-#line 1541 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1542 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 14:
-#line 241 "p-exp.y" /* yacc.c:1646  */
+#line 242 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap<predec_operation> (); }
-#line 1547 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1548 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 15:
-#line 246 "p-exp.y" /* yacc.c:1646  */
+#line 247 "p-exp.y" /* yacc.c:1646  */
     { search_field = 1; }
-#line 1553 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1554 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 16:
-#line 250 "p-exp.y" /* yacc.c:1646  */
+#line 251 "p-exp.y" /* yacc.c:1646  */
     {
 			  pstate->push_new<structop_operation>
 			    (pstate->pop (), copy_name ((yyvsp[0].sval)));
@@ -1567,11 +1568,11 @@ yyreduce:
 				current_type, (yyvsp[0].sval).ptr, 0);
 			    }
 			 }
-#line 1572 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1573 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 17:
-#line 268 "p-exp.y" /* yacc.c:1646  */
+#line 269 "p-exp.y" /* yacc.c:1646  */
     {
 			  pstate->push_new<structop_operation>
 			    (pstate->pop (), copy_name ((yyvsp[0].sval)));
@@ -1586,11 +1587,11 @@ yyreduce:
 				current_type, (yyvsp[0].sval).ptr, 0);
 			    }
 			}
-#line 1591 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1592 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 18:
-#line 284 "p-exp.y" /* yacc.c:1646  */
+#line 285 "p-exp.y" /* yacc.c:1646  */
     {
 			  structop_base_operation *op
 			    = new structop_ptr_operation (pstate->pop (),
@@ -1598,22 +1599,22 @@ yyreduce:
 			  pstate->mark_struct_expression (op);
 			  pstate->push (operation_up (op));
 			}
-#line 1603 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1604 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 19:
-#line 293 "p-exp.y" /* yacc.c:1646  */
+#line 294 "p-exp.y" /* yacc.c:1646  */
     {
 			  structop_base_operation *op
 			    = new structop_ptr_operation (pstate->pop (), "");
 			  pstate->mark_struct_expression (op);
 			  pstate->push (operation_up (op));
 			}
-#line 1614 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1615 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 20:
-#line 303 "p-exp.y" /* yacc.c:1646  */
+#line 304 "p-exp.y" /* yacc.c:1646  */
     { const char *arrayname;
 			  int arrayfieldindex
 			    = pascal_is_string_type (current_type, NULL, NULL,
@@ -1627,27 +1628,27 @@ yyreduce:
 				(pstate->pop (), arrayname);
 			    }
 			  push_current_type ();  }
-#line 1632 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1633 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 21:
-#line 317 "p-exp.y" /* yacc.c:1646  */
+#line 318 "p-exp.y" /* yacc.c:1646  */
     { pop_current_type ();
 			  pstate->wrap2<subscript_operation> ();
 			  if (current_type)
 			    current_type = current_type->target_type (); }
-#line 1641 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1642 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 22:
-#line 326 "p-exp.y" /* yacc.c:1646  */
+#line 327 "p-exp.y" /* yacc.c:1646  */
     { push_current_type ();
 			  pstate->start_arglist (); }
-#line 1648 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1649 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 23:
-#line 329 "p-exp.y" /* yacc.c:1646  */
+#line 330 "p-exp.y" /* yacc.c:1646  */
     {
 			  std::vector<operation_up> args
 			    = pstate->pop_vector (pstate->end_arglist ());
@@ -1657,23 +1658,23 @@ yyreduce:
 			  if (current_type)
 			    current_type = current_type->target_type ();
 			}
-#line 1662 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1663 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 25:
-#line 342 "p-exp.y" /* yacc.c:1646  */
+#line 343 "p-exp.y" /* yacc.c:1646  */
     { pstate->arglist_len = 1; }
-#line 1668 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1669 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 26:
-#line 344 "p-exp.y" /* yacc.c:1646  */
+#line 345 "p-exp.y" /* yacc.c:1646  */
     { pstate->arglist_len++; }
-#line 1674 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1675 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 27:
-#line 348 "p-exp.y" /* yacc.c:1646  */
+#line 349 "p-exp.y" /* yacc.c:1646  */
     { if (current_type)
 			    {
 			      /* Allow automatic dereference of classes.  */
@@ -1685,32 +1686,32 @@ yyreduce:
 			  pstate->push_new<unop_cast_operation>
 			    (pstate->pop (), (yyvsp[-3].tval));
 			  current_type = (yyvsp[-3].tval); }
-#line 1690 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1691 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 28:
-#line 362 "p-exp.y" /* yacc.c:1646  */
+#line 363 "p-exp.y" /* yacc.c:1646  */
     { }
-#line 1696 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1697 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 29:
-#line 368 "p-exp.y" /* yacc.c:1646  */
+#line 369 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<mul_operation> (); }
-#line 1702 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1703 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 30:
-#line 371 "p-exp.y" /* yacc.c:1646  */
+#line 372 "p-exp.y" /* yacc.c:1646  */
     {
 			  if (current_type && is_integral_type (current_type))
 			    leftdiv_is_integer = 1;
 			}
-#line 1711 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1712 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 31:
-#line 376 "p-exp.y" /* yacc.c:1646  */
+#line 377 "p-exp.y" /* yacc.c:1646  */
     {
 			  if (leftdiv_is_integer && current_type
 			      && is_integral_type (current_type))
@@ -1725,154 +1726,154 @@ yyreduce:
 
 			  pstate->wrap2<div_operation> ();
 			}
-#line 1730 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1731 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 32:
-#line 393 "p-exp.y" /* yacc.c:1646  */
+#line 394 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<intdiv_operation> (); }
-#line 1736 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1737 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 33:
-#line 397 "p-exp.y" /* yacc.c:1646  */
+#line 398 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<rem_operation> (); }
-#line 1742 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1743 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 34:
-#line 401 "p-exp.y" /* yacc.c:1646  */
+#line 402 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<add_operation> (); }
-#line 1748 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1749 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 35:
-#line 405 "p-exp.y" /* yacc.c:1646  */
+#line 406 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<sub_operation> (); }
-#line 1754 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1755 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 36:
-#line 409 "p-exp.y" /* yacc.c:1646  */
+#line 410 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<lsh_operation> (); }
-#line 1760 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1761 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 37:
-#line 413 "p-exp.y" /* yacc.c:1646  */
+#line 414 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<rsh_operation> (); }
-#line 1766 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1767 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 38:
-#line 417 "p-exp.y" /* yacc.c:1646  */
+#line 418 "p-exp.y" /* yacc.c:1646  */
     {
 			  pstate->wrap2<equal_operation> ();
 			  current_type = parse_type (pstate)->builtin_bool;
 			}
-#line 1775 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1776 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 39:
-#line 424 "p-exp.y" /* yacc.c:1646  */
+#line 425 "p-exp.y" /* yacc.c:1646  */
     {
 			  pstate->wrap2<notequal_operation> ();
 			  current_type = parse_type (pstate)->builtin_bool;
 			}
-#line 1784 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1785 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 40:
-#line 431 "p-exp.y" /* yacc.c:1646  */
+#line 432 "p-exp.y" /* yacc.c:1646  */
     {
 			  pstate->wrap2<leq_operation> ();
 			  current_type = parse_type (pstate)->builtin_bool;
 			}
-#line 1793 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1794 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 41:
-#line 438 "p-exp.y" /* yacc.c:1646  */
+#line 439 "p-exp.y" /* yacc.c:1646  */
     {
 			  pstate->wrap2<geq_operation> ();
 			  current_type = parse_type (pstate)->builtin_bool;
 			}
-#line 1802 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1803 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 42:
-#line 445 "p-exp.y" /* yacc.c:1646  */
+#line 446 "p-exp.y" /* yacc.c:1646  */
     {
 			  pstate->wrap2<less_operation> ();
 			  current_type = parse_type (pstate)->builtin_bool;
 			}
-#line 1811 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1812 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 43:
-#line 452 "p-exp.y" /* yacc.c:1646  */
+#line 453 "p-exp.y" /* yacc.c:1646  */
     {
 			  pstate->wrap2<gtr_operation> ();
 			  current_type = parse_type (pstate)->builtin_bool;
 			}
-#line 1820 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1821 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 44:
-#line 459 "p-exp.y" /* yacc.c:1646  */
+#line 460 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<bitwise_and_operation> (); }
-#line 1826 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1827 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 45:
-#line 463 "p-exp.y" /* yacc.c:1646  */
+#line 464 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<bitwise_xor_operation> (); }
-#line 1832 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1833 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 46:
-#line 467 "p-exp.y" /* yacc.c:1646  */
+#line 468 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<bitwise_ior_operation> (); }
-#line 1838 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1839 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 47:
-#line 471 "p-exp.y" /* yacc.c:1646  */
+#line 472 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap2<assign_operation> (); }
-#line 1844 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1845 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 48:
-#line 475 "p-exp.y" /* yacc.c:1646  */
+#line 476 "p-exp.y" /* yacc.c:1646  */
     {
 			  pstate->push_new<bool_operation> ((yyvsp[0].lval));
 			  current_type = parse_type (pstate)->builtin_bool;
 			}
-#line 1853 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1854 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 49:
-#line 482 "p-exp.y" /* yacc.c:1646  */
+#line 483 "p-exp.y" /* yacc.c:1646  */
     {
 			  pstate->push_new<bool_operation> ((yyvsp[0].lval));
 			  current_type = parse_type (pstate)->builtin_bool;
 			}
-#line 1862 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1863 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 50:
-#line 489 "p-exp.y" /* yacc.c:1646  */
+#line 490 "p-exp.y" /* yacc.c:1646  */
     {
 			  pstate->push_new<long_const_operation>
 			    ((yyvsp[0].typed_val_int).type, (yyvsp[0].typed_val_int).val);
 			  current_type = (yyvsp[0].typed_val_int).type;
 			}
-#line 1872 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1873 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 51:
-#line 497 "p-exp.y" /* yacc.c:1646  */
-    { YYSTYPE val;
+#line 498 "p-exp.y" /* yacc.c:1646  */
+    { p_exp_YYSTYPE val;
 			  parse_number (pstate, (yyvsp[0].ssym).stoken.ptr,
 					(yyvsp[0].ssym).stoken.length, 0, &val);
 			  pstate->push_new<long_const_operation>
@@ -1880,22 +1881,22 @@ yyreduce:
 			     val.typed_val_int.val);
 			  current_type = val.typed_val_int.type;
 			}
-#line 1885 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1886 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 52:
-#line 509 "p-exp.y" /* yacc.c:1646  */
+#line 510 "p-exp.y" /* yacc.c:1646  */
     {
 			  float_data data;
 			  std::copy (std::begin ((yyvsp[0].typed_val_float).val), std::end ((yyvsp[0].typed_val_float).val),
 				     std::begin (data));
 			  pstate->push_new<float_const_operation> ((yyvsp[0].typed_val_float).type, data);
 			}
-#line 1896 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1897 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 54:
-#line 521 "p-exp.y" /* yacc.c:1646  */
+#line 522 "p-exp.y" /* yacc.c:1646  */
     {
 			  pstate->push_dollar ((yyvsp[0].sval));
 
@@ -1919,29 +1920,29 @@ yyreduce:
 			      current_type = val->type ();
 			    }
  			}
-#line 1924 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1925 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 55:
-#line 547 "p-exp.y" /* yacc.c:1646  */
+#line 548 "p-exp.y" /* yacc.c:1646  */
     {
 			  current_type = parse_type (pstate)->builtin_int;
 			  (yyvsp[-1].tval) = check_typedef ((yyvsp[-1].tval));
 			  pstate->push_new<long_const_operation>
 			    (parse_type (pstate)->builtin_int,
 			     (yyvsp[-1].tval)->length ()); }
-#line 1935 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1936 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 56:
-#line 556 "p-exp.y" /* yacc.c:1646  */
+#line 557 "p-exp.y" /* yacc.c:1646  */
     { pstate->wrap<unop_sizeof_operation> ();
 			  current_type = parse_type (pstate)->builtin_int; }
-#line 1942 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1943 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 57:
-#line 560 "p-exp.y" /* yacc.c:1646  */
+#line 561 "p-exp.y" /* yacc.c:1646  */
     { /* C strings are converted into array constants with
 			     an explicit null byte added at the end.  Thus
 			     the array upper bound is the string length.
@@ -1960,11 +1961,11 @@ yyreduce:
 			  pstate->push_new<array_operation>
 			    (0, (yyvsp[0].sval).length, std::move (args));
 			}
-#line 1965 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1966 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 58:
-#line 582 "p-exp.y" /* yacc.c:1646  */
+#line 583 "p-exp.y" /* yacc.c:1646  */
     {
 			  struct value * this_val;
 			  struct type * this_type;
@@ -1987,11 +1988,11 @@ yyreduce:
 
 			  current_type = this_type;
 			}
-#line 1992 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 1993 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 59:
-#line 609 "p-exp.y" /* yacc.c:1646  */
+#line 610 "p-exp.y" /* yacc.c:1646  */
     {
 			  if ((yyvsp[0].ssym).sym.symbol != 0)
 			      (yyval.bval) = (yyvsp[0].ssym).sym.symbol->value_block ();
@@ -2008,42 +2009,43 @@ yyreduce:
 				       copy.c_str ());
 			    }
 			}
-#line 2013 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2014 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 60:
-#line 628 "p-exp.y" /* yacc.c:1646  */
+#line 629 "p-exp.y" /* yacc.c:1646  */
     {
 			  std::string copy = copy_name ((yyvsp[0].sval));
 			  struct symbol *tem
 			    = lookup_symbol (copy.c_str (), (yyvsp[-2].bval),
-					     VAR_DOMAIN, NULL).symbol;
+					     SEARCH_FUNCTION_DOMAIN,
+					     nullptr).symbol;
 
-			  if (!tem || tem->aclass () != LOC_BLOCK)
+			  if (tem == nullptr)
 			    error (_("No function \"%s\" in specified context."),
 				   copy.c_str ());
 			  (yyval.bval) = tem->value_block (); }
-#line 2028 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2030 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 61:
-#line 641 "p-exp.y" /* yacc.c:1646  */
+#line 643 "p-exp.y" /* yacc.c:1646  */
     { struct block_symbol sym;
 
 			  std::string copy = copy_name ((yyvsp[0].sval));
 			  sym = lookup_symbol (copy.c_str (), (yyvsp[-2].bval),
-					       VAR_DOMAIN, NULL);
+					       SEARCH_VFT, NULL);
 			  if (sym.symbol == 0)
 			    error (_("No symbol \"%s\" in specified context."),
 				   copy.c_str ());
 
 			  pstate->push_new<var_value_operation> (sym);
 			}
-#line 2044 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2046 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 62:
-#line 655 "p-exp.y" /* yacc.c:1646  */
+#line 657 "p-exp.y" /* yacc.c:1646  */
     {
 			  struct type *type = (yyvsp[-2].tval);
 
@@ -2055,24 +2057,24 @@ yyreduce:
 			  pstate->push_new<scope_operation>
 			    (type, copy_name ((yyvsp[0].sval)));
 			}
-#line 2060 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2062 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 64:
-#line 670 "p-exp.y" /* yacc.c:1646  */
+#line 672 "p-exp.y" /* yacc.c:1646  */
     {
 			  std::string name = copy_name ((yyvsp[0].sval));
 
 			  struct block_symbol sym
 			    = lookup_symbol (name.c_str (), nullptr,
-					     VAR_DOMAIN, nullptr);
+					     SEARCH_VFT, nullptr);
 			  pstate->push_symbol (name.c_str (), sym);
 			}
-#line 2073 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2075 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 65:
-#line 681 "p-exp.y" /* yacc.c:1646  */
+#line 683 "p-exp.y" /* yacc.c:1646  */
     { struct block_symbol sym = (yyvsp[0].ssym).sym;
 
 			  if (sym.symbol)
@@ -2127,65 +2129,65 @@ yyreduce:
 				       arg.c_str ());
 			    }
 			}
-#line 2132 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2134 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 68:
-#line 754 "p-exp.y" /* yacc.c:1646  */
+#line 756 "p-exp.y" /* yacc.c:1646  */
     { (yyval.tval) = lookup_pointer_type ((yyvsp[0].tval)); }
-#line 2138 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2140 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 69:
-#line 756 "p-exp.y" /* yacc.c:1646  */
+#line 758 "p-exp.y" /* yacc.c:1646  */
     { (yyval.tval) = (yyvsp[0].tsym).type; }
-#line 2144 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2146 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 70:
-#line 758 "p-exp.y" /* yacc.c:1646  */
+#line 760 "p-exp.y" /* yacc.c:1646  */
     { (yyval.tval)
 			    = lookup_struct (copy_name ((yyvsp[0].sval)).c_str (),
 					     pstate->expression_context_block);
 			}
-#line 2153 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2155 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 71:
-#line 763 "p-exp.y" /* yacc.c:1646  */
+#line 765 "p-exp.y" /* yacc.c:1646  */
     { (yyval.tval)
 			    = lookup_struct (copy_name ((yyvsp[0].sval)).c_str (),
 					     pstate->expression_context_block);
 			}
-#line 2162 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2164 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 72:
-#line 772 "p-exp.y" /* yacc.c:1646  */
+#line 774 "p-exp.y" /* yacc.c:1646  */
     { (yyval.sval) = (yyvsp[0].ssym).stoken; }
-#line 2168 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2170 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 73:
-#line 773 "p-exp.y" /* yacc.c:1646  */
+#line 775 "p-exp.y" /* yacc.c:1646  */
     { (yyval.sval) = (yyvsp[0].ssym).stoken; }
-#line 2174 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2176 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 74:
-#line 774 "p-exp.y" /* yacc.c:1646  */
+#line 776 "p-exp.y" /* yacc.c:1646  */
     { (yyval.sval) = (yyvsp[0].tsym).stoken; }
-#line 2180 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2182 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 75:
-#line 775 "p-exp.y" /* yacc.c:1646  */
+#line 777 "p-exp.y" /* yacc.c:1646  */
     { (yyval.sval) = (yyvsp[0].ssym).stoken; }
-#line 2186 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2188 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
 
-#line 2190 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2192 "p-exp.c.tmp" /* yacc.c:1646  */
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -2413,7 +2415,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 789 "p-exp.y" /* yacc.c:1906  */
+#line 791 "p-exp.y" /* yacc.c:1906  */
 
 
 /* Take care of parsing a number (anything that starts with a digit).
@@ -2424,7 +2426,7 @@ yyreturn:
 
 static int
 parse_number (struct parser_state *par_state,
-	      const char *p, int len, int parsed_float, YYSTYPE *putithere)
+	      const char *p, int len, int parsed_float, p_exp_YYSTYPE *putithere)
 {
   ULONGEST n = 0;
   ULONGEST prevn = 0;
@@ -2622,14 +2624,14 @@ pop_current_type (void)
     }
 }
 
-struct token
+struct p_token
 {
   const char *oper;
   int token;
   enum exp_opcode opcode;
 };
 
-static const struct token tokentab3[] =
+static const struct p_token tokentab3[] =
   {
     {"shr", RSH, OP_NULL},
     {"shl", LSH, OP_NULL},
@@ -2642,7 +2644,7 @@ static const struct token tokentab3[] =
     {"xor", XOR, OP_NULL}
   };
 
-static const struct token tokentab2[] =
+static const struct p_token tokentab2[] =
   {
     {"or", OR, OP_NULL},
     {"<>", NOTEQUAL, OP_NULL},
@@ -2669,6 +2671,28 @@ uptok (const char *tokstart, int namelen)
   return uptokstart;
 }
 
+/* Skip over a Pascal string.  STR must point to the opening single quote
+   character.  This function returns a pointer to the character after the
+   closing single quote character.
+
+   This function does not support embedded, escaped single quotes, which
+   is done by placing two consecutive single quotes into a string.
+   Support for this would be easy to add, but this function is only used
+   from the Python expression parser, and if we did skip over escaped
+   quotes then the rest of the expression parser wouldn't handle them
+   correctly.  */
+static const char *
+pascal_skip_string (const char *str)
+{
+  gdb_assert (*str == '\'');
+
+  do
+    ++str;
+  while (*str != '\0' && *str != '\'');
+
+  return str;
+}
+
 /* Read one token, getting characters through lexptr.  */
 
 static int
@@ -2747,7 +2771,7 @@ yylex (void)
       c = *pstate->lexptr++;
       if (c != '\'')
 	{
-	  namelen = skip_quoted (tokstart) - tokstart;
+	  namelen = pascal_skip_string (tokstart) - tokstart;
 	  if (namelen > 2)
 	    {
 	      pstate->lexptr = tokstart + namelen;
@@ -2787,7 +2811,7 @@ yylex (void)
 	  goto symbol;		/* Nope, must be a symbol.  */
 	}
 
-      /* FALL THRU.  */
+      [[fallthrough]];
 
     case '0':
     case '1':
@@ -3028,7 +3052,7 @@ yylex (void)
 	  static const char this_name[] = "this";
 
 	  if (lookup_symbol (this_name, pstate->expression_context_block,
-			     VAR_DOMAIN, NULL).symbol)
+			     SEARCH_VFT, NULL).symbol)
 	    {
 	      xfree (uptokstart);
 	      return THIS;
@@ -3068,7 +3092,7 @@ yylex (void)
       sym = NULL;
     else
       sym = lookup_symbol (tmp.c_str (), pstate->expression_context_block,
-			   VAR_DOMAIN, &is_a_field_of_this).symbol;
+			   SEARCH_VFT, &is_a_field_of_this).symbol;
     /* second chance uppercased (as Free Pascal does).  */
     if (!sym && is_a_field_of_this.type == NULL && !is_a_field)
       {
@@ -3084,7 +3108,7 @@ yylex (void)
 	 sym = NULL;
        else
 	 sym = lookup_symbol (tmp.c_str (), pstate->expression_context_block,
-			      VAR_DOMAIN, &is_a_field_of_this).symbol;
+			      SEARCH_VFT, &is_a_field_of_this).symbol;
       }
     /* Third chance Capitalized (as GPC does).  */
     if (!sym && is_a_field_of_this.type == NULL && !is_a_field)
@@ -3107,7 +3131,7 @@ yylex (void)
 	 sym = NULL;
        else
 	 sym = lookup_symbol (tmp.c_str (), pstate->expression_context_block,
-			      VAR_DOMAIN, &is_a_field_of_this).symbol;
+			      SEARCH_VFT, &is_a_field_of_this).symbol;
       }
 
     if (is_a_field || (is_a_field_of_this.type != NULL))
@@ -3204,7 +3228,7 @@ yylex (void)
 		      cur_sym
 			= lookup_symbol (ncopy,
 					 pstate->expression_context_block,
-					 VAR_DOMAIN, NULL).symbol;
+					 SEARCH_VFT, NULL).symbol;
 		      if (cur_sym)
 			{
 			  if (cur_sym->aclass () == LOC_TYPEDEF)
@@ -3248,7 +3272,7 @@ yylex (void)
 	&& ((tokstart[0] >= 'a' && tokstart[0] < 'a' + input_radix - 10)
 	    || (tokstart[0] >= 'A' && tokstart[0] < 'A' + input_radix - 10)))
       {
- 	YYSTYPE newlval;	/* Its value is ignored.  */
+ 	p_exp_YYSTYPE newlval;	/* Its value is ignored.  */
 	hextype = parse_number (pstate, tokstart, namelen, 0, &newlval);
 	if (hextype == INT)
 	  {
@@ -3288,8 +3312,5 @@ pascal_language::parser (struct parser_state *par_state) const
 static void
 yyerror (const char *msg)
 {
-  if (pstate->prev_lexptr)
-    pstate->lexptr = pstate->prev_lexptr;
-
-  error (_("A %s in expression, near `%s'."), msg, pstate->lexptr);
+  pstate->parse_error (msg);
 }
