@@ -1,6 +1,6 @@
 /* Ada Ravenscar thread support.
 
-   Copyright (C) 2004-2023 Free Software Foundation, Inc.
+   Copyright (C) 2004-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,7 +17,6 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 #include "gdbcore.h"
 #include "gdbthread.h"
 #include "ada-lang.h"
@@ -383,7 +382,7 @@ get_running_thread_id (int cpu)
   gdb_byte *buf;
   CORE_ADDR object_addr;
   struct type *builtin_type_void_data_ptr
-    = builtin_type (target_gdbarch ())->builtin_data_ptr;
+    = builtin_type (current_inferior ()->arch ())->builtin_data_ptr;
 
   if (!object_msym.minsym)
     return 0;
@@ -648,7 +647,8 @@ ravenscar_thread_target::get_fpu_state (struct regcache *regcache,
   if (fpu_context.minsym == nullptr)
     return NO_FP_REGISTERS;
 
-  struct type *ptr_type = builtin_type (target_gdbarch ())->builtin_data_ptr;
+  type *ptr_type
+    = builtin_type (current_inferior ()->arch ())->builtin_data_ptr;
   ptr_type = lookup_pointer_type (ptr_type);
   value *val = value_from_pointer (ptr_type, fpu_context.value_address ());
 
@@ -684,7 +684,7 @@ ravenscar_thread_target::fetch_registers (struct regcache *regcache,
       struct gdbarch *gdbarch = regcache->arch ();
       bool is_active = task_is_currently_active (ptid);
       struct ravenscar_arch_ops *arch_ops = gdbarch_ravenscar_ops (gdbarch);
-      gdb::optional<fpu_state> fp_state;
+      std::optional<fpu_state> fp_state;
 
       int low_reg = regnum == -1 ? 0 : regnum;
       int high_reg = regnum == -1 ? gdbarch_num_regs (gdbarch) : regnum + 1;
@@ -730,7 +730,7 @@ ravenscar_thread_target::store_registers (struct regcache *regcache,
       struct gdbarch *gdbarch = regcache->arch ();
       bool is_active = task_is_currently_active (ptid);
       struct ravenscar_arch_ops *arch_ops = gdbarch_ravenscar_ops (gdbarch);
-      gdb::optional<fpu_state> fp_state;
+      std::optional<fpu_state> fp_state;
 
       int low_reg = regnum == -1 ? 0 : regnum;
       int high_reg = regnum == -1 ? gdbarch_num_regs (gdbarch) : regnum + 1;
@@ -874,7 +874,7 @@ ravenscar_inferior_created (inferior *inf)
   const char *err_msg;
 
   if (!ravenscar_task_support
-      || gdbarch_ravenscar_ops (target_gdbarch ()) == NULL
+      || gdbarch_ravenscar_ops (current_inferior ()->arch ()) == NULL
       || !has_ravenscar_runtime ())
     return;
 
