@@ -1,4 +1,4 @@
-/* Copyright (C) 2012-2023 Free Software Foundation, Inc.
+/* Copyright (C) 2012-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -15,7 +15,6 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 #include "osabi.h"
 #include "regcache.h"
 #include "gdbcore.h"
@@ -93,7 +92,7 @@ rs6000_lynx178_push_dummy_call (struct gdbarch *gdbarch,
       int reg_size = register_size (gdbarch, ii + 3);
 
       arg = args[argno];
-      type = check_typedef (value_type (arg));
+      type = check_typedef (arg->type ());
       len = type->length ();
 
       if (type->code () == TYPE_CODE_FLT)
@@ -111,7 +110,7 @@ rs6000_lynx178_push_dummy_call (struct gdbarch *gdbarch,
 
 	  gdb_assert (len <= 8);
 
-	  target_float_convert (value_contents (arg).data (), type, reg_val,
+	  target_float_convert (arg->contents ().data (), type, reg_val,
 				reg_type);
 	  regcache->cooked_write (fp_regnum, reg_val);
 	  ++f_argno;
@@ -126,7 +125,7 @@ rs6000_lynx178_push_dummy_call (struct gdbarch *gdbarch,
 	      gdb_byte word[PPC_MAX_REGISTER_SIZE];
 	      memset (word, 0, reg_size);
 	      memcpy (word,
-		      ((char *) value_contents (arg).data ()) + argbytes,
+		      ((char *) arg->contents ().data ()) + argbytes,
 		      (len - argbytes) > reg_size
 			? reg_size : len - argbytes);
 	      regcache->cooked_write (tdep->ppc_gp0_regnum + 3 + ii, word);
@@ -144,7 +143,7 @@ rs6000_lynx178_push_dummy_call (struct gdbarch *gdbarch,
 	  gdb_byte word[PPC_MAX_REGISTER_SIZE];
 
 	  memset (word, 0, reg_size);
-	  memcpy (word, value_contents (arg).data (), len);
+	  memcpy (word, arg->contents ().data (), len);
 	  regcache->cooked_write (tdep->ppc_gp0_regnum + 3 +ii, word);
 	}
       ++argno;
@@ -184,7 +183,7 @@ ran_out_of_registers_for_arguments:
 	{
 	  struct value *val = args[jj];
 
-	  space += align_up (value_type (val)->length (), 4);
+	  space += align_up (val->type ()->length (), 4);
 	}
 
       /* Add location required for the rest of the parameters.  */
@@ -206,7 +205,7 @@ ran_out_of_registers_for_arguments:
       if (argbytes)
 	{
 	  write_memory (sp + 24 + (ii * 4),
-			value_contents (arg).data () + argbytes,
+			arg->contents ().data () + argbytes,
 			len - argbytes);
 	  ++argno;
 	  ii += align_up (len - argbytes, 4) / 4;
@@ -217,7 +216,7 @@ ran_out_of_registers_for_arguments:
 	{
 
 	  arg = args[argno];
-	  type = check_typedef (value_type (arg));
+	  type = check_typedef (arg->type ());
 	  len = type->length ();
 
 
@@ -229,11 +228,11 @@ ran_out_of_registers_for_arguments:
 	      gdb_assert (len <= 8);
 
 	      regcache->cooked_write (tdep->ppc_fp0_regnum + 1 + f_argno,
-				      value_contents (arg).data ());
+				      arg->contents ().data ());
 	      ++f_argno;
 	    }
 
-	  write_memory (sp + 24 + (ii * 4), value_contents (arg).data (), len);
+	  write_memory (sp + 24 + (ii * 4), arg->contents ().data (), len);
 	  ii += align_up (len, 4) / 4;
 	}
     }
