@@ -1,5 +1,5 @@
 /* Handle TIC6X (DSBT) shared libraries for GDB, the GNU Debugger.
-   Copyright (C) 2010-2023 Free Software Foundation, Inc.
+   Copyright (C) 2010-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,19 +17,17 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 
-#include "defs.h"
 #include "inferior.h"
 #include "gdbcore.h"
 #include "solib.h"
 #include "solist.h"
 #include "objfiles.h"
 #include "symtab.h"
-#include "language.h"
 #include "command.h"
-#include "gdbcmd.h"
-#include "elf-bfd.h"
 #include "gdb_bfd.h"
 #include "solib-dsbt.h"
+#include "elf/common.h"
+#include "cli/cli-cmds.h"
 
 #define GOT_MODULE_OFFSET 4
 
@@ -124,7 +122,7 @@ struct dbst_ext_link_map
 
 /* Link map info to include in an allocated so_list entry */
 
-struct lm_info_dsbt : public lm_info_base
+struct lm_info_dsbt final : public lm_info
 {
   ~lm_info_dsbt ()
   {
@@ -167,19 +165,17 @@ struct dsbt_info
 /* Per-program-space data key.  */
 static const registry<program_space>::key<dsbt_info> solib_dsbt_pspace_data;
 
-/* Get the current dsbt data.  If none is found yet, add it now.  This
+/* Get the dsbt solib data for PSPACE.  If none is found yet, add it now.  This
    function always returns a valid object.  */
 
-static struct dsbt_info *
-get_dsbt_info (void)
+static dsbt_info *
+get_dsbt_info (program_space *pspace)
 {
-  struct dsbt_info *info;
-
-  info = solib_dsbt_pspace_data.get (current_program_space);
-  if (info != NULL)
+  dsbt_info *info = solib_dsbt_pspace_data.get (pspace);
+  if (info != nullptr)
     return info;
 
-  return solib_dsbt_pspace_data.emplace (current_program_space);
+  return solib_dsbt_pspace_data.emplace (pspace);
 }
 
 
@@ -198,14 +194,16 @@ dsbt_print_loadmap (struct int_elf32_dsbt_loadmap *map)
 
       for (i = 0; i < map->nsegs; i++)
 	gdb_printf ("%s:%s -> %s:%s\n",
-		    print_core_address (target_gdbarch (),
+		    print_core_address (current_inferior ()->arch (),
 					map->segs[i].p_vaddr),
-		    print_core_address (target_gdbarch (),
-					map->segs[i].p_vaddr
-					+ map->segs[i].p_memsz),
-		    print_core_address (target_gdbarch (), map->segs[i].addr),
-		    print_core_address (target_gdbarch (), map->segs[i].addr
-					+ map->segs[i].p_memsz));
+		    print_core_address (current_inferior ()->arch (),
+					(map->segs[i].p_vaddr
+					 + map->segs[i].p_memsz)),
+		    print_core_address (current_inferior ()->arch (),
+					map->segs[i].addr),
+		    print_core_address (current_inferior ()->arch (),
+					(map->segs[i].addr
+					 + map->segs[i].p_memsz)));
     }
 }
 
@@ -214,7 +212,7 @@ dsbt_print_loadmap (struct int_elf32_dsbt_loadmap *map)
 static struct int_elf32_dsbt_loadmap *
 decode_loadmap (const gdb_byte *buf)
 {
-  enum bfd_endian byte_order = gdbarch_byte_order (target_gdbarch ());
+  bfd_endian byte_order = gdbarch_byte_order (current_inferior ()->arch ());
   const struct ext_elf32_dsbt_loadmap *ext_ldmbuf;
   struct int_elf32_dsbt_loadmap *int_ldmbuf;
 
@@ -269,9 +267,6 @@ decode_loadmap (const gdb_byte *buf)
   return int_ldmbuf;
 }
 
-
-static struct dsbt_info *get_dsbt_info (void);
-
 /* Interrogate the Linux kernel to find out where the program was loaded.
    There are two load maps; one for the executable and one for the
    interpreter (only in the case of a dynamically linked executable).  */
@@ -279,8 +274,8 @@ static struct dsbt_info *get_dsbt_info (void);
 static void
 dsbt_get_initial_loadmaps (void)
 {
-  struct dsbt_info *info = get_dsbt_info ();
-  gdb::optional<gdb::byte_vector> buf
+  dsbt_info *info = get_dsbt_info (current_program_space);
+  std::optional<gdb::byte_vector> buf
     = target_read_alloc (current_inferior ()->top_target (),
 			 TARGET_OBJECT_FDPIC, "exec");
 
@@ -314,7 +309,7 @@ dsbt_get_initial_loadmaps (void)
 static struct int_elf32_dsbt_loadmap *
 fetch_loadmap (CORE_ADDR ldmaddr)
 {
-  enum bfd_endian byte_order = gdbarch_byte_order (target_gdbarch ());
+  bfd_endian byte_order = gdbarch_byte_order (current_inferior ()->arch ());
   struct ext_elf32_dsbt_loadmap ext_ldmbuf_partial;
   struct ext_elf32_dsbt_loadmap *ext_ldmbuf;
   struct int_elf32_dsbt_loadmap *int_ldmbuf;
@@ -437,11 +432,11 @@ displacement_from_map (struct int_elf32_dsbt_loadmap *map,
 static CORE_ADDR
 lm_base (void)
 {
-  enum bfd_endian byte_order = gdbarch_byte_order (target_gdbarch ());
+  bfd_endian byte_order = gdbarch_byte_order (current_inferior ()->arch ());
   struct bound_minimal_symbol got_sym;
   CORE_ADDR addr;
   gdb_byte buf[TIC6X_PTR_SIZE];
-  struct dsbt_info *info = get_dsbt_info ();
+  dsbt_info *info = get_dsbt_info (current_program_space);
 
   /* One of our assumptions is that the main executable has been relocated.
      Bail out if this has not happened.  (Note that post_create_inferior
@@ -507,23 +502,22 @@ lm_base (void)
 }
 
 
-/* Build a list of `struct so_list' objects describing the shared
+/* Build a list of `struct solib' objects describing the shared
    objects currently loaded in the inferior.  This list does not
    include an entry for the main executable file.
 
    Note that we only gather information directly available from the
    inferior --- we don't examine any of the shared library files
-   themselves.  The declaration of `struct so_list' says which fields
+   themselves.  The declaration of `struct solib' says which fields
    we provide values for.  */
 
-static struct so_list *
+static intrusive_list<solib>
 dsbt_current_sos (void)
 {
-  enum bfd_endian byte_order = gdbarch_byte_order (target_gdbarch ());
+  bfd_endian byte_order = gdbarch_byte_order (current_inferior ()->arch ());
   CORE_ADDR lm_addr;
-  struct so_list *sos_head = NULL;
-  struct so_list **sos_next_ptr = &sos_head;
-  struct dsbt_info *info = get_dsbt_info ();
+  dsbt_info *info = get_dsbt_info (current_program_space);
+  intrusive_list<solib> sos;
 
   /* Make sure that the main executable has been relocated.  This is
      required in order to find the address of the global offset table,
@@ -537,7 +531,8 @@ dsbt_current_sos (void)
      dsbt_current_sos, and also precedes the call to
      solib_create_inferior_hook.   (See post_create_inferior in
      infcmd.c.)  */
-  if (info->main_executable_lm_info == 0 && core_bfd != NULL)
+  if (info->main_executable_lm_info == 0
+      && current_program_space->core_bfd () != nullptr)
     dsbt_relocate_main_executable ();
 
   /* Locate the address of the first link map struct.  */
@@ -588,7 +583,6 @@ dsbt_current_sos (void)
       if (dsbt_index != 0)
 	{
 	  struct int_elf32_dsbt_loadmap *loadmap;
-	  struct so_list *sop;
 	  CORE_ADDR addr;
 
 	  loadmap = fetch_loadmap (map_addr);
@@ -599,9 +593,8 @@ dsbt_current_sos (void)
 	      break;
 	    }
 
-	  sop = XCNEW (struct so_list);
-	  lm_info_dsbt *li = new lm_info_dsbt;
-	  sop->lm_info = li;
+	  solib *sop = new solib;
+	  auto li = std::make_unique<lm_info_dsbt> ();
 	  li->map = loadmap;
 	  /* Fetch the name.  */
 	  addr = extract_unsigned_integer (lm_buf.l_name,
@@ -618,13 +611,11 @@ dsbt_current_sos (void)
 		gdb_printf (gdb_stdlog, "current_sos: name = %s\n",
 			    name_buf.get ());
 
-	      strncpy (sop->so_name, name_buf.get (), SO_NAME_MAX_PATH_SIZE - 1);
-	      sop->so_name[SO_NAME_MAX_PATH_SIZE - 1] = '\0';
-	      strcpy (sop->so_original_name, sop->so_name);
+	      sop->so_name = name_buf.get ();
+	      sop->so_original_name = sop->so_name;
 	    }
 
-	  *sos_next_ptr = sop;
-	  sos_next_ptr = &sop->next;
+	  sos.push_back (*sop);
 	}
       else
 	{
@@ -635,7 +626,7 @@ dsbt_current_sos (void)
 					  sizeof (lm_buf.l_next), byte_order);
     }
 
-  return sos_head;
+  return sos;
 }
 
 /* Return 1 if PC lies in the dynamic symbol resolution code of the
@@ -644,7 +635,7 @@ dsbt_current_sos (void)
 static int
 dsbt_in_dynsym_resolve_code (CORE_ADDR pc)
 {
-  struct dsbt_info *info = get_dsbt_info ();
+  dsbt_info *info = get_dsbt_info (current_program_space);
 
   return ((pc >= info->interp_text_sect_low && pc < info->interp_text_sect_high)
 	  || (pc >= info->interp_plt_sect_low && pc < info->interp_plt_sect_high)
@@ -662,14 +653,6 @@ enable_break_failure_warning (void)
 	     "and track explicitly loaded dynamic code."));
 }
 
-/* Helper function for gdb_bfd_lookup_symbol.  */
-
-static int
-cmp_name (const asymbol *sym, const void *data)
-{
-  return (strcmp (sym->name, (const char *) data) == 0);
-}
-
 /* The dynamic linkers has, as part of its debugger interface, support
    for arranging for the inferior to hit a breakpoint after mapping in
    the shared libraries.  This function enables that breakpoint.
@@ -682,7 +665,6 @@ static int
 enable_break (void)
 {
   asection *interp_sect;
-  struct dsbt_info *info;
 
   if (current_program_space->exec_bfd () == NULL)
     return 0;
@@ -690,7 +672,7 @@ enable_break (void)
   if (!target_has_execution ())
     return 0;
 
-  info = get_dsbt_info ();
+  dsbt_info *info = get_dsbt_info (current_program_space);
 
   info->interp_text_sect_low = 0;
   info->interp_text_sect_high = 0;
@@ -759,8 +741,13 @@ enable_break (void)
 	    = info->interp_plt_sect_low + bfd_section_size (interp_sect);
 	}
 
-      addr = gdb_bfd_lookup_symbol (tmp_bfd.get (), cmp_name,
-				    "_dl_debug_state");
+      addr = (gdb_bfd_lookup_symbol
+	      (tmp_bfd.get (),
+	       [] (const asymbol *sym)
+	       {
+		 return strcmp (sym->name, "_dl_debug_state") == 0;
+	       }));
+
       if (addr != 0)
 	{
 	  if (solib_dsbt_debug)
@@ -775,7 +762,7 @@ enable_break (void)
 			hex_string_custom (addr, 8));
 
 	  /* Now (finally!) create the solib breakpoint.  */
-	  create_solib_event_breakpoint (target_gdbarch (), addr);
+	  create_solib_event_breakpoint (current_inferior ()->arch (), addr);
 
 	  ret = 1;
 	}
@@ -805,8 +792,7 @@ dsbt_relocate_main_executable (void)
 {
   struct int_elf32_dsbt_loadmap *ldm;
   int changed;
-  struct obj_section *osect;
-  struct dsbt_info *info = get_dsbt_info ();
+  dsbt_info *info = get_dsbt_info (current_program_space);
 
   dsbt_get_initial_loadmaps ();
   ldm = info->exec_loadmap;
@@ -819,13 +805,13 @@ dsbt_relocate_main_executable (void)
   section_offsets new_offsets (objf->section_offsets.size ());
   changed = 0;
 
-  ALL_OBJFILE_OSECTIONS (objf, osect)
+  for (obj_section *osect : objf->sections ())
     {
       CORE_ADDR orig_addr, addr, offset;
       int osect_idx;
       int seg;
 
-      osect_idx = osect - objf->sections;
+      osect_idx = osect - objf->sections_start;
 
       /* Current address of section.  */
       addr = osect->addr ();
@@ -857,7 +843,7 @@ dsbt_relocate_main_executable (void)
 }
 
 /* When gdb starts up the inferior, it nurses it along (through the
-   shell) until it is ready to execute it's first instruction.  At this
+   shell) until it is ready to execute its first instruction.  At this
    point, this function gets called via solib_create_inferior_hook.
 
    For the DSBT shared library, the main executable needs to be relocated.
@@ -878,9 +864,9 @@ dsbt_solib_create_inferior_hook (int from_tty)
 }
 
 static void
-dsbt_clear_solib (void)
+dsbt_clear_solib (program_space *pspace)
 {
-  struct dsbt_info *info = get_dsbt_info ();
+  dsbt_info *info = get_dsbt_info (pspace);
 
   info->lm_base_cache = 0;
   info->main_lm_addr = 0;
@@ -890,19 +876,10 @@ dsbt_clear_solib (void)
 }
 
 static void
-dsbt_free_so (struct so_list *so)
-{
-  lm_info_dsbt *li = (lm_info_dsbt *) so->lm_info;
-
-  delete li;
-}
-
-static void
-dsbt_relocate_section_addresses (struct so_list *so,
-				 struct target_section *sec)
+dsbt_relocate_section_addresses (solib &so, target_section *sec)
 {
   int seg;
-  lm_info_dsbt *li = (lm_info_dsbt *) so->lm_info;
+  auto *li = gdb::checked_static_cast<lm_info_dsbt *> (so.lm_info.get ());
   int_elf32_dsbt_loadmap *map = li->map;
 
   for (seg = 0; seg < map->nsegs; seg++)
@@ -925,10 +902,9 @@ show_dsbt_debug (struct ui_file *file, int from_tty,
   gdb_printf (file, _("solib-dsbt debugging is %s.\n"), value);
 }
 
-const struct target_so_ops dsbt_so_ops =
+const solib_ops dsbt_so_ops =
 {
   dsbt_relocate_section_addresses,
-  dsbt_free_so,
   nullptr,
   dsbt_clear_solib,
   dsbt_solib_create_inferior_hook,
