@@ -1,6 +1,6 @@
 /* This testcase is part of GDB, the GNU debugger.
 
-   Copyright 2022-2023 Free Software Foundation, Inc.
+   Copyright 2022-2024 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,9 +17,16 @@
 
 extern int foo (void);
 
+int
+bar (void)
+{
+  asm ("bar_label: .globl bar_label");
+  return 0;
+}
+
 int
 main (void)
 {
-  int res = foo ();
+  int res = foo () + bar ();
   return res;
 }
