@@ -1,6 +1,6 @@
 /* General window behavior.
 
-   Copyright (C) 1998-2022 Free Software Foundation, Inc.
+   Copyright (C) 1998-2024 Free Software Foundation, Inc.
 
    Contributed by Hewlett-Packard Company.
 
@@ -19,13 +19,12 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 #include "tui/tui.h"
 #include "tui/tui-data.h"
 #include "tui/tui-io.h"
 #include "tui/tui-wingeneral.h"
 #include "tui/tui-win.h"
-#include "tui/tui-stack.h"
+#include "tui/tui-status.h"
 #include "cli/cli-style.h"
 
 #include "gdb_curses.h"
@@ -77,7 +76,7 @@ tui_win_info::refresh_window ()
     tui_wrefresh (handle.get ());
 }
 
-/* Draw a border arround the window.  */
+/* Draw a border around the window.  */
 static void
 box_win (struct tui_win_info *win_info, 
 	 bool highlight_flag)
@@ -98,27 +97,23 @@ box_win (struct tui_win_info *win_info,
 			   ? tui_active_border_style.style ()
 			   : tui_border_style.style ()));
   wattron (win, attrs);
-#ifdef HAVE_WBORDER
   wborder (win, tui_border_vline, tui_border_vline,
 	   tui_border_hline, tui_border_hline,
 	   tui_border_ulcorner, tui_border_urcorner,
 	   tui_border_llcorner, tui_border_lrcorner);
-#else
-  box (win, tui_border_vline, tui_border_hline);
-#endif
-  if (!win_info->title.empty ())
+  if (!win_info->title ().empty ())
     {
       /* Emit "+-TITLE-+" -- so 2 characters on the right and 2 on
 	 the left.  */
-      int max_len = win_info->width - 2 - 2;
+      int max_len = win_info->width - win_info->box_size () - 2;
 
-      if (win_info->title.size () <= max_len)
-	mvwaddstr (win, 0, 2, win_info->title.c_str ());
+      if (win_info->title ().size () <= max_len)
+	mvwaddstr (win, 0, 2, win_info->title ().c_str ());
       else
 	{
 	  std::string truncated
-	    = "..." + win_info->title.substr (win_info->title.size ()
-					      - max_len + 3);
+	    = "..." + win_info->title ().substr (win_info->title ().size ()
+						 - max_len + 3);
 	  mvwaddstr (win, 0, 2, truncated.c_str ());
 	}
     }
