@@ -1,5 +1,5 @@
 /* <proc_service.h> replacement for systems that don't have it.
-   Copyright (C) 2000-2022 Free Software Foundation, Inc.
+   Copyright (C) 2000-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -25,11 +25,11 @@
 
 /* glibc's proc_service.h doesn't wrap itself with extern "C".  Need
    to do it ourselves.  */
-EXTERN_C_PUSH
+extern "C" {
 
 #include <proc_service.h>
 
-EXTERN_C_POP
+}
 
 #else /* HAVE_PROC_SERVICE_H */
 
@@ -69,7 +69,7 @@ EXTERN_C_POP
 # endif
 #endif
 
-EXTERN_C_PUSH
+extern "C" {
 
 /* Functions in this interface return one of these status codes.  */
 typedef enum
@@ -166,7 +166,7 @@ extern ps_err_e ps_lsetxregs (struct ps_prochandle *ph, lwpid_t lwpid,
 /* Log a message (sends to gdb_stderr).  */
 extern void ps_plog (const char *fmt, ...);
 
-EXTERN_C_POP
+}
 
 #endif /* HAVE_PROC_SERVICE_H */
 
