@@ -1,5 +1,5 @@
 /* CPU support.
-   Copyright (C) 1998-2022 Free Software Foundation, Inc.
+   Copyright (C) 1998-2024 Free Software Foundation, Inc.
    Contributed by Cygnus Solutions.
 
 This file is part of GDB, the GNU debugger.
@@ -31,12 +31,17 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 /* ??? wip.  better solution must wait.  */
 
 SIM_RC
-sim_cpu_alloc_all (SIM_DESC sd, int ncpus)
+sim_cpu_alloc_all_extra (SIM_DESC sd, int ncpus, size_t extra_bytes)
 {
   int c;
 
+  /* TODO: This should be a command line option for users to control.  */
+  if (ncpus == 0)
+    ncpus = MAX_NR_PROCESSORS;
+
   for (c = 0; c < ncpus; ++c)
-    STATE_CPU (sd, c) = sim_cpu_alloc (sd);
+    STATE_CPU (sd, c) = sim_cpu_alloc_extra (sd, extra_bytes);
+
   return SIM_RC_OK;
 }
 
@@ -44,15 +49,18 @@ sim_cpu_alloc_all (SIM_DESC sd, int ncpus)
    EXTRA_BYTES is additional space to allocate for the sim_cpu struct.  */
 
 sim_cpu *
-sim_cpu_alloc (SIM_DESC sd)
+sim_cpu_alloc_extra (SIM_DESC sd, size_t extra_bytes)
 {
-  int extra_bytes = 0;
+  sim_cpu *cpu = zalloc (sizeof (*cpu));
 
-#ifdef CGEN_ARCH
-  extra_bytes += cgen_cpu_max_extra_bytes (sd);
+#ifndef CGEN_ARCH
+# define cgen_cpu_max_extra_bytes(sd) 0
 #endif
+  extra_bytes += cgen_cpu_max_extra_bytes (sd);
+  if (extra_bytes)
+    CPU_ARCH_DATA (cpu) = zalloc (extra_bytes);
 
-  return zalloc (sizeof (sim_cpu) + extra_bytes);
+  return cpu;
 }
 
 /* Free all resources held by all cpus.  */
@@ -72,6 +80,7 @@ sim_cpu_free_all (SIM_DESC sd)
 void
 sim_cpu_free (sim_cpu *cpu)
 {
+  free (CPU_ARCH_DATA (cpu));
   free (cpu);
 }
 
