@@ -1,6 +1,6 @@
 /* gdb-if.c -- sim interface to GDB.
 
-Copyright (C) 2011-2023 Free Software Foundation, Inc.
+Copyright (C) 2011-2024 Free Software Foundation, Inc.
 Contributed by Red Hat, Inc.
 
 This file is part of the GNU simulators.
@@ -205,32 +205,32 @@ sim_create_inferior (SIM_DESC sd, struct bfd *abfd,
 /* Read memory.  */
 
 uint64_t
-sim_read (SIM_DESC sd, uint64_t mem, void *buf, uint64_t length)
+sim_read (SIM_DESC sd, uint64_t addr, void *buf, uint64_t length)
 {
   check_desc (sd);
 
-  if (mem >= MEM_SIZE)
+  if (addr >= MEM_SIZE)
     return 0;
-  else if (mem + length > MEM_SIZE)
-    length = MEM_SIZE - mem;
+  else if (addr + length > MEM_SIZE)
+    length = MEM_SIZE - addr;
 
-  mem_get_blk (mem, buf, length);
+  mem_get_blk (addr, buf, length);
   return length;
 }
 
 /* Write memory.  */
 
 uint64_t
-sim_write (SIM_DESC sd, uint64_t mem, const void *buf, uint64_t length)
+sim_write (SIM_DESC sd, uint64_t addr, const void *buf, uint64_t length)
 {
   check_desc (sd);
 
-  if (mem >= MEM_SIZE)
+  if (addr >= MEM_SIZE)
     return 0;
-  else if (mem + length > MEM_SIZE)
-    length = MEM_SIZE - mem;
+  else if (addr + length > MEM_SIZE)
+    length = MEM_SIZE - addr;
 
-  mem_put_blk (mem, buf, length);
+  mem_put_blk (addr, buf, length);
   return length;
 }
 
